@@ -1,30 +1,33 @@
 // bsonelement.h
 
-/*    Copyright 2009 10gen Inc.
+
+/**
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #pragma once
@@ -237,11 +240,12 @@ public:
         return type() == EOO;
     }
 
-    /** Size of the element.
-        @param maxLen If maxLen is specified, don't scan more than maxLen bytes to calculate size.
-    */
-    int size(int maxLen) const;
-    int size() const;
+    /**
+     * Size of the element.
+     */
+    int size() const {
+        return totalSize;
+    }
 
     /** Wrap this element up as a singleton object. */
     BSONObj wrap() const;
@@ -263,8 +267,6 @@ public:
      * NOTE: size includes the NULL terminator.
      */
     int fieldNameSize() const {
-        if (fieldNameSize_ == -1)
-            fieldNameSize_ = (int)strlen(fieldName()) + 1;
         return fieldNameSize_;
     }
 
@@ -653,43 +655,41 @@ public:
     }
 
     // @param maxLen don't scan more than maxLen bytes
-    explicit BSONElement(const char* d, int maxLen) : data(d) {
+    explicit BSONElement(const char* d) : data(d) {
         if (eoo()) {
-            totalSize = 1;
             fieldNameSize_ = 0;
+            totalSize = 1;
         } else {
-            totalSize = -1;
-            fieldNameSize_ = -1;
-            if (maxLen != -1) {
-                size_t size = strnlen(fieldName(), maxLen - 1);
-                uassert(10333, "Invalid field name", size < size_t(maxLen - 1));
-                fieldNameSize_ = size + 1;
+            fieldNameSize_ = strlen(d + 1 /*skip type*/) + 1 /*include NUL byte*/;
+            totalSize = computeSize();
+        }
+    }
+
+    struct CachedSizeTag {};  // Opts in to next constructor.
+
+    /**
+     * Construct a BSONElement where you already know the length of the name and/or the total size
+     * of the element. fieldNameSize includes the null terminator. You may pass -1 for either or
+     * both sizes to indicate that they are unknown and should be computed.
+     */
+    BSONElement(const char* d, int fieldNameSize, int totalSize, CachedSizeTag) : data(d) {
+        if (eoo()) {
+            fieldNameSize_ = 0;
+            this->totalSize = 1;
+        } else {
+            if (fieldNameSize == -1) {
+                fieldNameSize_ = strlen(d + 1 /*skip type*/) + 1 /*include NUL byte*/;
+            } else {
+                fieldNameSize_ = fieldNameSize;
+            }
+            if (totalSize == -1) {
+                this->totalSize = computeSize();
+            } else {
+                this->totalSize = totalSize;
             }
         }
     }
 
-    explicit BSONElement(const char* d) : data(d) {
-        fieldNameSize_ = -1;
-        totalSize = -1;
-        if (eoo()) {
-            fieldNameSize_ = 0;
-            totalSize = 1;
-        }
-    }
-
-    struct FieldNameSizeTag {};  // For disambiguation with ctor taking 'maxLen' above.
-
-    /** Construct a BSONElement where you already know the length of the name. The value
-     *  passed here includes the null terminator. The data pointed to by 'd' must not
-     *  represent an EOO. You may pass -1 to indicate that you don't actually know the
-     *  size.
-     */
-    BSONElement(const char* d, int fieldNameSize, FieldNameSizeTag)
-        : data(d),
-          fieldNameSize_(fieldNameSize)  // internal size includes null terminator
-          ,
-          totalSize(-1) {}
-
     std::string _asCode() const;
 
     template <typename T>
@@ -697,11 +697,11 @@ public:
 
 private:
     const char* data;
-    mutable int fieldNameSize_;  // cached value
-
-    mutable int totalSize; /* caches the computed size */
+    int fieldNameSize_;  // internal size includes null terminator
+    int totalSize;
 
     friend class BSONObjIterator;
+    friend class BSONObjStlIterator;
     friend class BSONObj;
     const BSONElement& chk(BSONType t) const {
         if (t != type()) {
@@ -714,6 +714,9 @@ private:
         }
         return *this;
     }
+
+    // Only called from constructors.
+    int computeSize() const;
 };
 
 inline bool BSONElement::trueValue() const {
