@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -75,6 +77,8 @@ struct CollModRequest {
     std::string collValidationLevel = {};
     BSONElement usePowerOf2Sizes = {};
     BSONElement noPadding = {};
+    // Indicates whether any of the above fields have been set.
+    bool isEmptyCollModRequest = true;
 };
 
 StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
@@ -82,13 +86,13 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
                                                Collection* coll,
                                                const BSONObj& cmdObj,
                                                BSONObjBuilder* oplogEntryBuilder) {
-
     bool isView = !coll;
 
     CollModRequest cmr;
 
     BSONForEach(e, cmdObj) {
         const auto fieldName = e.fieldNameStringData();
+
         if (Command::isGenericArgument(fieldName)) {
             continue;  // Don't add to oplog builder.
         } else if (fieldName == "collMod") {
@@ -173,6 +177,7 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
                               "existing expireAfterSeconds field is not a number");
             }
 
+            cmr.isEmptyCollModRequest = false;
         } else if (fieldName == "validator" && !isView) {
             MatchExpressionParser::AllowedFeatureSet allowedFeatures =
                 MatchExpressionParser::kBanAllSpecialFeatures;
@@ -200,18 +205,21 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
             }
 
             cmr.collValidator = e;
+            cmr.isEmptyCollModRequest = false;
         } else if (fieldName == "validationLevel" && !isView) {
             auto statusW = coll->parseValidationLevel(e.String());
             if (!statusW.isOK())
                 return statusW.getStatus();
 
             cmr.collValidationLevel = e.String();
+            cmr.isEmptyCollModRequest = false;
         } else if (fieldName == "validationAction" && !isView) {
             auto statusW = coll->parseValidationAction(e.String());
             if (!statusW.isOK())
                 return statusW.getStatus();
 
             cmr.collValidationAction = e.String();
+            cmr.isEmptyCollModRequest = false;
         } else if (fieldName == "pipeline") {
             if (!isView) {
                 return Status(ErrorCodes::InvalidOptions,
@@ -221,6 +229,8 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
                 return Status(ErrorCodes::InvalidOptions, "not a valid aggregation pipeline");
             }
             cmr.viewPipeLine = e;
+            cmr.isEmptyCollModRequest = false;
+
         } else if (fieldName == "viewOn") {
             if (!isView) {
                 return Status(ErrorCodes::InvalidOptions,
@@ -230,6 +240,7 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
                 return Status(ErrorCodes::InvalidOptions, "'viewOn' option must be a string");
             }
             cmr.viewOn = e.str();
+            cmr.isEmptyCollModRequest = false;
         } else {
             if (isView) {
                 return Status(ErrorCodes::InvalidOptions,
@@ -246,6 +257,7 @@ StatusWith<CollModRequest> parseCollModRequest(OperationContext* opCtx,
             else
                 return Status(ErrorCodes::InvalidOptions,
                               str::stream() << "unknown option to collMod: " << fieldName);
+            cmr.isEmptyCollModRequest = false;
         }
 
         oplogEntryBuilder->append(e);
@@ -341,9 +353,12 @@ Status _collModInternal(OperationContext* opCtx,
     if (!statusW.isOK()) {
         return statusW.getStatus();
     }
-
     CollModRequest cmr = statusW.getValue();
 
+    if (cmr.isEmptyCollModRequest && !upgradeUUID) {
+        return Status::OK();
+    }
+
     WriteUnitOfWork wunit(opCtx);
 
     // Handle collMod on a view and return early. The View Catalog handles the creation of oplog
@@ -396,6 +411,9 @@ Status _collModInternal(OperationContext* opCtx,
             // Notify the index catalog that the definition of this index changed.
             cmr.idx = coll->getIndexCatalog()->refreshEntry(opCtx, cmr.idx);
             result->appendAs(newExpireSecs, "expireAfterSeconds_new");
+            opCtx->recoveryUnit()->onRollback([ opCtx, idx = cmr.idx, coll ]() {
+                coll->getIndexCatalog()->refreshEntry(opCtx, idx);
+            });
         }
 
         // Save previous TTL index expiration.
@@ -404,17 +422,13 @@ Status _collModInternal(OperationContext* opCtx,
                                  cmr.idx->indexName()};
     }
 
-    // Validator
+    // The Validator, ValidationAction and ValidationLevel are already parsed and must be OK.
     if (!cmr.collValidator.eoo())
-        coll->setValidator(opCtx, cmr.collValidator.Obj()).transitional_ignore();
-
-    // ValidationAction
+        invariantOK(coll->setValidator(opCtx, cmr.collValidator.Obj()));
     if (!cmr.collValidationAction.empty())
-        coll->setValidationAction(opCtx, cmr.collValidationAction).transitional_ignore();
-
-    // ValidationLevel
+        invariantOK(coll->setValidationAction(opCtx, cmr.collValidationAction));
     if (!cmr.collValidationLevel.empty())
-        coll->setValidationLevel(opCtx, cmr.collValidationLevel).transitional_ignore();
+        invariantOK(coll->setValidationLevel(opCtx, cmr.collValidationLevel));
 
     // UsePowerof2Sizes
     if (!cmr.usePowerOf2Sizes.eoo())
