@@ -1,23 +1,25 @@
-/*-
- *    Copyright (C) 2017 MongoDB Inc.
+
+/**
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -42,6 +44,7 @@
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/background.h"
 #include "mongo/db/catalog/collection_catalog_entry.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/database_catalog_entry.h"
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog/index_consistency.h"
@@ -132,7 +135,7 @@ std::unique_ptr<CollatorInterface> parseCollation(OperationContext* opCtx,
 
     return std::move(collator.getValue());
 }
-}
+}  // namespace
 
 using std::unique_ptr;
 using std::endl;
@@ -198,6 +201,15 @@ CollectionImpl::~CollectionImpl() {
     _magic = 0;
 }
 
+void CollectionImpl::refreshUUID(OperationContext* opCtx) {
+    auto options = getCatalogEntry()->getCollectionOptions(opCtx);
+    // refreshUUID may be called from outside a WriteUnitOfWork. In such cases, there is no
+    // change to any on disk data, so no rollback handler is needed.
+    if (opCtx->lockState()->inAWriteUnitOfWork())
+        opCtx->recoveryUnit()->onRollback([ this, oldUUID = _uuid ] { this->_uuid = oldUUID; });
+    _uuid = options.uuid;
+}
+
 bool CollectionImpl::requiresIdIndex() const {
     if (_ns.isVirtualized() || _ns.isOplog()) {
         // No indexes on virtual collections or the oplog.
@@ -505,11 +517,16 @@ Status CollectionImpl::_insertDocuments(OperationContext* opCtx,
     return status;
 }
 
+bool CollectionImpl::haveCappedWaiters() {
+    // Waiters keep a shared_ptr to '_cappedNotifier', so there are waiters if this CollectionImpl's
+    // shared_ptr is not unique (use_count > 1).
+    return _cappedNotifier.use_count() > 1;
+}
+
 void CollectionImpl::notifyCappedWaitersIfNeeded() {
     // If there is a notifier object and another thread is waiting on it, then we notify
-    // waiters of this document insert. Waiters keep a shared_ptr to '_cappedNotifier', so
-    // there are waiters if this CollectionImpl's shared_ptr is not unique (use_count > 1).
-    if (_cappedNotifier && !_cappedNotifier.unique())
+    // waiters of this document insert.
+    if (haveCappedWaiters())
         _cappedNotifier->notifyAll();
 }
 
@@ -909,6 +926,14 @@ Status CollectionImpl::setValidator(OperationContext* opCtx, BSONObj validatorDo
 
     _details->updateValidator(opCtx, validatorDoc, getValidationLevel(), getValidationAction());
 
+    opCtx->recoveryUnit()->onRollback([
+        this,
+        oldValidator = std::move(_validator),
+        oldValidatorDoc = std::move(_validatorDoc)
+    ]() mutable {
+        this->_validator = std::move(oldValidator);
+        this->_validatorDoc = std::move(oldValidatorDoc);
+    });
     _validator = std::move(statusWithMatcher.getValue());
     _validatorDoc = std::move(validatorDoc);
     return Status::OK();
@@ -974,9 +999,12 @@ Status CollectionImpl::setValidationLevel(OperationContext* opCtx, StringData ne
         return status.getStatus();
     }
 
+    auto oldValidationLevel = _validationLevel;
     _validationLevel = status.getValue();
 
     _details->updateValidator(opCtx, _validatorDoc, getValidationLevel(), getValidationAction());
+    opCtx->recoveryUnit()->onRollback(
+        [this, oldValidationLevel]() { this->_validationLevel = oldValidationLevel; });
 
     return Status::OK();
 }
@@ -989,9 +1017,12 @@ Status CollectionImpl::setValidationAction(OperationContext* opCtx, StringData n
         return status.getStatus();
     }
 
+    auto oldValidationAction = _validationAction;
     _validationAction = status.getValue();
 
     _details->updateValidator(opCtx, _validatorDoc, getValidationLevel(), getValidationAction());
+    opCtx->recoveryUnit()->onRollback(
+        [this, oldValidationAction]() { this->_validationAction = oldValidationAction; });
 
     return Status::OK();
 }
@@ -1002,6 +1033,19 @@ Status CollectionImpl::updateValidator(OperationContext* opCtx,
                                        StringData newAction) {
     invariant(opCtx->lockState()->isCollectionLockedForMode(ns().toString(), MODE_X));
 
+    opCtx->recoveryUnit()->onRollback([
+        this,
+        oldValidator = std::move(_validator),
+        oldValidatorDoc = std::move(_validatorDoc),
+        oldValidationLevel = _validationLevel,
+        oldValidationAction = _validationAction
+    ]() mutable {
+        this->_validator = std::move(oldValidator);
+        this->_validatorDoc = std::move(oldValidatorDoc);
+        this->_validationLevel = oldValidationLevel;
+        this->_validationAction = oldValidationAction;
+    });
+
     _details->updateValidator(opCtx, newValidator, newLevel, newAction);
     _validatorDoc = std::move(newValidator);
 
@@ -1184,6 +1228,54 @@ void _reportValidationResults(OperationContext* opCtx,
         output->append("indexDetails", indexDetails->done());
     }
 }
+template <typename T>
+void addErrorIfUnequal(T stored, T cached, StringData name, ValidateResults* results) {
+    if (stored != cached) {
+        results->valid = false;
+        results->errors.push_back(str::stream() << "stored value for " << name
+                                                << " does not match cached value: "
+                                                << stored
+                                                << " != "
+                                                << cached);
+    }
+}
+
+void _validateCatalogEntry(OperationContext* opCtx,
+                           CollectionImpl* coll,
+                           BSONObj validatorDoc,
+                           ValidateResults* results) {
+    CollectionOptions options = coll->getCatalogEntry()->getCollectionOptions(opCtx);
+    addErrorIfUnequal(options.uuid, coll->uuid(), "UUID", results);
+    const CollatorInterface* collation = coll->getDefaultCollator();
+    addErrorIfUnequal(options.collation.isEmpty(), !collation, "simple collation", results);
+    if (!options.collation.isEmpty() && collation)
+        addErrorIfUnequal(options.collation.toString(),
+                          collation->getSpec().toBSON().toString(),
+                          "collation",
+                          results);
+    addErrorIfUnequal(options.capped, coll->isCapped(), "is capped", results);
+
+    addErrorIfUnequal(options.validator.toString(), validatorDoc.toString(), "validator", results);
+    if (!options.validator.isEmpty() && !validatorDoc.isEmpty()) {
+        addErrorIfUnequal(options.validationAction.length() ? options.validationAction : "error",
+                          coll->getValidationAction().toString(),
+                          "validation action",
+                          results);
+        addErrorIfUnequal(options.validationLevel.length() ? options.validationLevel : "strict",
+                          coll->getValidationLevel().toString(),
+                          "validation level",
+                          results);
+    }
+
+    addErrorIfUnequal(options.isView(), false, "is a view", results);
+    auto status = options.validateForStorage();
+    if (!status.isOK()) {
+        results->valid = false;
+        results->errors.push_back(str::stream() << "collection options are not valid for storage: "
+                                                << options.toBSON());
+    }
+}
+
 }  // namespace
 
 Status CollectionImpl::validate(OperationContext* opCtx,
@@ -1202,12 +1294,14 @@ Status CollectionImpl::validate(OperationContext* opCtx,
         RecordStoreValidateAdaptor indexValidator = RecordStoreValidateAdaptor(
             opCtx, &indexConsistency, level, &_indexCatalog, &indexNsResultsMap);
 
-
         // Validate the record store
         log(LogComponent::kIndex) << "validating collection " << ns().toString() << endl;
         _validateRecordStore(
             opCtx, _recordStore, level, background, &indexValidator, results, output);
 
+        // Validate in-memory catalog information with the persisted info.
+        _validateCatalogEntry(opCtx, this, _validatorDoc, results);
+
         // Validate indexes and check for mismatches.
         if (results->valid) {
             _validateIndexes(opCtx,
