@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -68,6 +70,46 @@ NamespaceString getNamespaceFromUUIDElement(OperationContext* opCtx, const BSONE
     return getNamespaceFromUUID(opCtx, uuid);
 }
 
+Status renameTargetCollectionToTmp(OperationContext* opCtx,
+                                   const NamespaceString& sourceNs,
+                                   const UUID& sourceUUID,
+                                   Database* const targetDB,
+                                   const NamespaceString& targetNs,
+                                   const UUID& targetUUID) {
+    repl::UnreplicatedWritesBlock uwb(opCtx);
+
+    auto tmpNameResult = targetDB->makeUniqueCollectionNamespace(opCtx, "tmp%%%%%.rename");
+    if (!tmpNameResult.isOK()) {
+        return tmpNameResult.getStatus().withContext(
+            str::stream() << "Cannot generate a temporary collection name for the target "
+                          << targetNs.ns()
+                          << " ("
+                          << targetUUID
+                          << ") so that the source"
+                          << sourceNs.ns()
+                          << " ("
+                          << sourceUUID
+                          << ") could be renamed to "
+                          << targetNs.ns());
+    }
+    const auto& tmpName = tmpNameResult.getValue();
+    const bool stayTemp = true;
+    return writeConflictRetry(opCtx, "renameCollection", targetNs.ns(), [&] {
+        WriteUnitOfWork wunit(opCtx);
+        auto status = targetDB->renameCollection(opCtx, targetNs.ns(), tmpName.ns(), stayTemp);
+        if (!status.isOK())
+            return status;
+
+        wunit.commit();
+
+        log() << "Successfully renamed the target " << targetNs.ns() << " (" << targetUUID
+              << ") to " << tmpName << " so that the source " << sourceNs.ns() << " (" << sourceUUID
+              << ") could be renamed to " << targetNs.ns();
+
+        return Status::OK();
+    });
+}
+
 Status renameCollectionCommon(OperationContext* opCtx,
                               const NamespaceString& source,
                               const NamespaceString& target,
@@ -75,8 +117,8 @@ Status renameCollectionCommon(OperationContext* opCtx,
                               repl::OpTime renameOpTimeFromApplyOps,
                               const RenameCollectionOptions& options) {
     auto uuidString = targetUUID ? targetUUID->toString() : "no UUID";
-    log() << "renameCollection: renaming collection " << source << " to " << target << " ("
-          << uuidString << ")";
+    log() << "renameCollection: renaming collection " << uuidString << " from " << source << " to "
+          << target;
 
     // A valid 'renameOpTimeFromApplyOps' is not allowed when writes are replicated.
     if (!renameOpTimeFromApplyOps.isNull() && opCtx->writesAreReplicated()) {
@@ -144,16 +186,10 @@ Status renameCollectionCommon(OperationContext* opCtx,
     Collection* targetColl = targetDB->getCollection(opCtx, target);
     if (targetColl) {
         // If we already have the collection with the target UUID, we found our future selves,
-        // so nothing left to do but drop the source collection in case of cross-db renames.
+        // so nothing left to do.
         if (targetUUID && targetUUID == targetColl->uuid()) {
-            if (source.db() == target.db())
-                return Status::OK();
-            BSONObjBuilder unusedResult;
-            return dropCollection(opCtx,
-                                  source,
-                                  unusedResult,
-                                  {},
-                                  DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
+            invariant(source == target);
+            return Status::OK();
         }
         if (CollectionShardingState::get(opCtx, target)->getMetadata()) {
             return {ErrorCodes::IllegalOperation, "cannot rename to a sharded collection"};
@@ -163,11 +199,34 @@ Status renameCollectionCommon(OperationContext* opCtx,
             return Status(ErrorCodes::NamespaceExists, "target namespace exists");
         }
 
+        // If UUID doesn't point to the existing target, we should rename the target rather than
+        // drop it.
+        if (options.dropTargetUUID && options.dropTargetUUID != targetColl->uuid()) {
+            auto dropTargetNssFromUUID = getNamespaceFromUUID(opCtx, options.dropTargetUUID.get());
+            // We need to rename the targetColl to a temporary name.
+            auto status = renameTargetCollectionToTmp(
+                opCtx, source, targetUUID.get(), targetDB, target, targetColl->uuid().get());
+            if (!status.isOK())
+                return status;
+            targetColl = nullptr;
+        }
     } else if (targetDB->getViewCatalog()->lookup(opCtx, target.ns())) {
         return Status(ErrorCodes::NamespaceExists,
                       str::stream() << "a view already exists with that name: " << target.ns());
     }
 
+    // When reapplying oplog entries (such as in the case of initial sync) we need
+    // to identify the collection to drop by UUID, as otherwise we might end up
+    // dropping the wrong collection.
+    if (!targetColl && options.dropTargetUUID) {
+        invariant(options.dropTarget);
+        auto dropTargetNssFromUUID = getNamespaceFromUUID(opCtx, options.dropTargetUUID.get());
+        if (!dropTargetNssFromUUID.isEmpty() && !dropTargetNssFromUUID.isDropPendingNamespace()) {
+            invariant(dropTargetNssFromUUID.db() == target.db());
+            targetColl = targetDB->getCollection(opCtx, dropTargetNssFromUUID);
+        }
+    }
+
     auto sourceUUID = sourceColl->uuid();
     // If we are renaming in the same database, just rename the namespace and we're done.
     if (sourceDB == targetDB) {
@@ -185,8 +244,11 @@ Status renameCollectionCommon(OperationContext* opCtx,
                         return status;
                     }
                 }
+                // We have to override the provided 'dropTarget' setting for idempotency reasons to
+                // avoid unintentionally removing a collection on a secondary with the same name as
+                // the target.
                 opObserver->onRenameCollection(
-                    opCtx, source, target, sourceUUID, options.dropTarget, {}, stayTemp);
+                    opCtx, source, target, sourceUUID, false, {}, stayTemp);
                 wunit.commit();
                 return Status::OK();
             }
@@ -194,6 +256,53 @@ Status renameCollectionCommon(OperationContext* opCtx,
             // Target collection exists - drop it.
             invariant(options.dropTarget);
             auto dropTargetUUID = targetColl->uuid();
+
+            // If this rename collection is replicated, check for long index names in the target
+            // collection that may exceed the MMAPv1 namespace limit when the target collection
+            // is renamed with a drop-pending namespace.
+            auto replCoord = repl::ReplicationCoordinator::get(opCtx);
+            auto isOplogDisabledForNamespace = replCoord->isOplogDisabledFor(opCtx, target);
+            auto isMasterSlave =
+                repl::ReplicationCoordinator::modeMasterSlave == replCoord->getReplicationMode();
+            if (!isOplogDisabledForNamespace && !isMasterSlave) {
+                invariant(opCtx->writesAreReplicated());
+                invariant(renameOpTimeFromApplyOps.isNull());
+
+                // Compile a list of any indexes that would become too long following the
+                // drop-pending rename. In the case that this collection drop gets rolled back, this
+                // will incur a performance hit, since those indexes will have to be rebuilt from
+                // scratch, but data integrity is maintained.
+                std::vector<IndexDescriptor*> indexesToDrop;
+                auto indexIter = targetColl->getIndexCatalog()->getIndexIterator(opCtx, true);
+
+                // Determine which index names are too long. Since we don't have the collection
+                // rename optime at this time, use the maximum optime to check the index names.
+                auto longDpns = target.makeDropPendingNamespace(repl::OpTime::max());
+                while (indexIter.more()) {
+                    auto index = indexIter.next();
+                    auto status = longDpns.checkLengthForRename(index->indexName().size());
+                    if (!status.isOK()) {
+                        indexesToDrop.push_back(index);
+                    }
+                }
+
+                // Drop the offending indexes.
+                auto sourceUuidString = sourceUUID ? sourceUUID.get().toString() : "no UUID";
+                auto dropTargetUuidString =
+                    dropTargetUUID ? dropTargetUUID.get().toString() : "no UUID";
+                for (auto&& index : indexesToDrop) {
+                    log() << "renameCollection: renaming collection " << sourceUuidString
+                          << " from " << source << " to " << target << " (" << dropTargetUuidString
+                          << ") - target collection contains an index namespace '"
+                          << index->indexNamespace()
+                          << "' that would be too long after drop-pending rename. Dropping index "
+                             "immediately.";
+                    fassertStatusOK(50941, targetColl->getIndexCatalog()->dropIndex(opCtx, index));
+                    opObserver->onDropIndex(
+                        opCtx, target, targetColl->uuid(), index->indexName(), index->infoObj());
+                }
+            }
+
             auto renameOpTime = opObserver->onRenameCollection(
                 opCtx, source, target, sourceUUID, true, dropTargetUUID, options.stayTemp);
 
@@ -212,7 +321,7 @@ Status renameCollectionCommon(OperationContext* opCtx,
             // No logOp necessary because the entire renameCollection command is one logOp.
             repl::UnreplicatedWritesBlock uwb(opCtx);
 
-            status = targetDB->dropCollection(opCtx, target.ns(), renameOpTime);
+            status = targetDB->dropCollection(opCtx, targetColl->ns().ns(), renameOpTime);
             if (!status.isOK()) {
                 return status;
             }
@@ -444,22 +553,48 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
         sourceNss = uiNss;
     }
 
-    if (sourceNss.isDropPendingNamespace()) {
-        return Status(ErrorCodes::NamespaceNotFound,
-                      str::stream() << "renameCollectionForApplyOps() cannot accept a source "
-                                       "collection that is in a drop-pending state: "
-                                    << sourceNss.toString()
-                                    << " (UUID: "
-                                    << ui.toString(false)
-                                    << ")");
-    }
-
     OptionalCollectionUUID targetUUID;
     if (!ui.eoo())
         targetUUID = uassertStatusOK(UUID::parse(ui));
 
     RenameCollectionOptions options;
     options.dropTarget = cmd["dropTarget"].trueValue();
+    if (cmd["dropTarget"].type() == BinData) {
+        auto uuid = uassertStatusOK(UUID::parse(cmd["dropTarget"]));
+        options.dropTargetUUID = uuid;
+    }
+
+    const Collection* const sourceColl =
+        AutoGetCollectionForRead(opCtx, sourceNss, AutoGetCollection::ViewMode::kViewsPermitted)
+            .getCollection();
+
+    if (sourceNss.isDropPendingNamespace() || sourceColl == nullptr) {
+        NamespaceString dropTargetNss;
+
+        if (options.dropTarget)
+            dropTargetNss = targetNss;
+
+        if (options.dropTargetUUID) {
+            dropTargetNss = getNamespaceFromUUID(opCtx, options.dropTargetUUID.get());
+        }
+
+        // Downgrade renameCollection to dropCollection.
+        if (!dropTargetNss.isEmpty()) {
+            BSONObjBuilder unusedResult;
+            return dropCollection(opCtx,
+                                  dropTargetNss,
+                                  unusedResult,
+                                  renameOpTime,
+                                  DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
+        }
+
+        return Status(ErrorCodes::NamespaceNotFound,
+                      str::stream()
+                          << "renameCollection() cannot accept a source "
+                             "collection that does not exist or is in a drop-pending state: "
+                          << sourceNss.toString());
+    }
+
     options.stayTemp = cmd["stayTemp"].trueValue();
     return renameCollectionCommon(opCtx, sourceNss, targetNss, targetUUID, renameOpTime, options);
 }
