@@ -1,23 +1,25 @@
+
 /**
- *    Copyright 2017 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -36,6 +38,7 @@
 #include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/catalog/index_create.h"
 #include "mongo/db/catalog/rename_collection.h"
+#include "mongo/db/catalog/uuid_catalog.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
@@ -86,7 +89,8 @@ public:
                             Collection* coll,
                             const NamespaceString& collectionName,
                             const CollectionOptions& options,
-                            const BSONObj& idIndex) override;
+                            const BSONObj& idIndex,
+                            const OplogSlot& createOpTime) override;
 
     repl::OpTime onDropCollection(OperationContext* opCtx,
                                   const NamespaceString& collectionName,
@@ -108,6 +112,7 @@ public:
     bool onInsertsIsGlobalWriteLockExclusive = false;
 
     bool onRenameCollectionCalled = false;
+    bool onRenameCollectionDropTarget = false;
     repl::OpTime renameOpTime = {Timestamp(Seconds(100), 1U), 1LL};
 
 private:
@@ -151,9 +156,10 @@ void OpObserverMock::onCreateCollection(OperationContext* opCtx,
                                         Collection* coll,
                                         const NamespaceString& collectionName,
                                         const CollectionOptions& options,
-                                        const BSONObj& idIndex) {
+                                        const BSONObj& idIndex,
+                                        const OplogSlot& createOpTime) {
     _logOp(opCtx, collectionName, "create");
-    OpObserverNoop::onCreateCollection(opCtx, coll, collectionName, options, idIndex);
+    OpObserverNoop::onCreateCollection(opCtx, coll, collectionName, options, idIndex, createOpTime);
 }
 
 repl::OpTime OpObserverMock::onDropCollection(OperationContext* opCtx,
@@ -174,6 +180,7 @@ repl::OpTime OpObserverMock::onRenameCollection(OperationContext* opCtx,
     OpObserverNoop::onRenameCollection(
         opCtx, fromCollection, toCollection, uuid, dropTarget, dropTargetUUID, stayTemp);
     onRenameCollectionCalled = true;
+    onRenameCollectionDropTarget = dropTarget;
     return renameOpTime;
 }
 
@@ -284,6 +291,15 @@ CollectionOptions _makeCollectionOptionsWithUuid() {
     return options;
 }
 
+/**
+ * Creates a collection with UUID and returns the UUID.
+ */
+CollectionUUID _createCollectionWithUUID(OperationContext* opCtx, const NamespaceString& nss) {
+    const auto options = _makeCollectionOptionsWithUuid();
+    _createCollection(opCtx, nss, options);
+    return options.uuid.get();
+}
+
 /**
  * Returns true if collection exists.
  */
@@ -312,6 +328,14 @@ CollectionUUID _getCollectionUuid(OperationContext* opCtx, const NamespaceString
     return *(options.uuid);
 }
 
+/**
+ * Get collection namespace by UUID.
+ */
+NamespaceString _getCollectionNssFromUUID(OperationContext* opCtx, const UUID& uuid) {
+    Collection* source = UUIDCatalog::get(opCtx).lookupCollectionByUUID(uuid);
+    return source ? source->ns() : NamespaceString();
+}
+
 /**
  * Returns true if namespace refers to a temporary collection.
  */
@@ -500,6 +524,141 @@ TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsAcrossDatabaseWithTarget
     ASSERT_EQUALS(uuid, _getCollectionUuid(_opCtx.get(), _targetNssDifferentDb));
 }
 
+TEST_F(RenameCollectionTest, RenameCollectionToItselfByNsForApplyOps) {
+    auto dbName = _sourceNss.db().toString();
+    auto uuid = _createCollectionWithUUID(_opCtx.get(), _sourceNss);
+    auto uuidDoc = BSON("ui" << uuid);
+    auto cmd = BSON("renameCollection" << _sourceNss.ns() << "to" << _sourceNss.ns() << "dropTarget"
+                                       << true);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _sourceNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionToItselfByUUIDForApplyOps) {
+    auto dbName = _targetNss.db().toString();
+    auto uuid = _createCollectionWithUUID(_opCtx.get(), _targetNss);
+    auto uuidDoc = BSON("ui" << uuid);
+    auto cmd = BSON("renameCollection" << _sourceNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                       << true);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionByUUIDRatherThanNsForApplyOps) {
+    auto realRenameFromNss = NamespaceString("test.bar2");
+    auto dbName = realRenameFromNss.db().toString();
+    auto uuid = _createCollectionWithUUID(_opCtx.get(), realRenameFromNss);
+    auto uuidDoc = BSON("ui" << uuid);
+    auto cmd = BSON("renameCollection" << _sourceNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                       << true);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetByUUIDTargetDoesNotExist) {
+    const auto& collA = NamespaceString("test.A");
+    const auto& collB = NamespaceString("test.B");
+    const auto& collC = NamespaceString("test.C");
+    auto dbName = collA.db().toString();
+    auto collAUUID = _createCollectionWithUUID(_opCtx.get(), collA);
+    auto collCUUID = _createCollectionWithUUID(_opCtx.get(), collC);
+    auto uuidDoc = BSON("ui" << collAUUID);
+    // Rename A to B, drop C, where B is not an existing collection
+    auto cmd =
+        BSON("renameCollection" << collA.ns() << "to" << collB.ns() << "dropTarget" << collCUUID);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    // A and C should be dropped
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collA));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collC));
+    // B (originally A) should exist
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), collB));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetByUUIDTargetExists) {
+    const auto& collA = NamespaceString("test.A");
+    const auto& collB = NamespaceString("test.B");
+    const auto& collC = NamespaceString("test.C");
+    auto dbName = collA.db().toString();
+    auto collAUUID = _createCollectionWithUUID(_opCtx.get(), collA);
+    auto collBUUID = _createCollectionWithUUID(_opCtx.get(), collB);
+    auto collCUUID = _createCollectionWithUUID(_opCtx.get(), collC);
+    auto uuidDoc = BSON("ui" << collAUUID);
+    // Rename A to B, drop C, where B is an existing collection
+    // B should be kept but with a temporary name
+    auto cmd =
+        BSON("renameCollection" << collA.ns() << "to" << collB.ns() << "dropTarget" << collCUUID);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    // A and C should be dropped
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collA));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collC));
+    // B (originally A) should exist
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), collB));
+    // The original B should exist too, but with a temporary name
+    const auto& tmpB = UUIDCatalog::get(_opCtx.get()).lookupNSSByUUID(collBUUID);
+    ASSERT_FALSE(tmpB.isEmpty());
+    ASSERT_TRUE(tmpB.coll().startsWith("tmp"));
+    ASSERT_TRUE(tmpB != collB);
+}
+
+TEST_F(RenameCollectionTest,
+       RenameCollectionForApplyOpsDropTargetByUUIDTargetExistsButTemporarily) {
+
+    const auto& collA = NamespaceString("test.A");
+    const auto& collB = NamespaceString("test.B");
+    const auto& collC = NamespaceString("test.C");
+
+    CollectionOptions collectionOptions = _makeCollectionOptionsWithUuid();
+    collectionOptions.temp = true;
+    _createCollection(_opCtx.get(), collB, collectionOptions);
+    auto collBUUID = _getCollectionUuid(_opCtx.get(), collB);
+
+    auto dbName = collA.db().toString();
+    auto collAUUID = _createCollectionWithUUID(_opCtx.get(), collA);
+    auto collCUUID = _createCollectionWithUUID(_opCtx.get(), collC);
+    auto uuidDoc = BSON("ui" << collAUUID);
+    // Rename A to B, drop C, where B is an existing collection
+    // B should be kept but with a temporary name
+    auto cmd =
+        BSON("renameCollection" << collA.ns() << "to" << collB.ns() << "dropTarget" << collCUUID);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    // A and C should be dropped
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collA));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collC));
+    // B (originally A) should exist
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), collB));
+    // The original B should exist too, but with a temporary name
+    const auto& tmpB = UUIDCatalog::get(_opCtx.get()).lookupNSSByUUID(collBUUID);
+    ASSERT_FALSE(tmpB.isEmpty());
+    ASSERT_TRUE(tmpB != collB);
+    ASSERT_TRUE(tmpB.coll().startsWith("tmp"));
+    ASSERT_TRUE(_isTempCollection(_opCtx.get(), tmpB));
+}
+
+TEST_F(RenameCollectionTest,
+       RenameCollectionForApplyOpsDropTargetByUUIDTargetExistsButRealDropTargetDoesNotExist) {
+    const auto& collA = NamespaceString("test.A");
+    const auto& collB = NamespaceString("test.B");
+    auto dbName = collA.db().toString();
+    auto collAUUID = _createCollectionWithUUID(_opCtx.get(), collA);
+    auto collBUUID = _createCollectionWithUUID(_opCtx.get(), collB);
+    auto collCUUID = UUID::gen();
+    auto uuidDoc = BSON("ui" << collAUUID);
+    // Rename A to B, drop C, where B is an existing collection
+    // B should be kept but with a temporary name
+    auto cmd =
+        BSON("renameCollection" << collA.ns() << "to" << collB.ns() << "dropTarget" << collCUUID);
+    ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, uuidDoc["ui"], cmd, {}));
+    // A and C should be dropped
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), collA));
+    // B (originally A) should exist
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), collB));
+    // The original B should exist too, but with a temporary name
+    const auto& tmpB = UUIDCatalog::get(_opCtx.get()).lookupNSSByUUID(collBUUID);
+    ASSERT_FALSE(tmpB.isEmpty());
+    ASSERT_TRUE(tmpB != collB);
+    ASSERT_TRUE(tmpB.coll().startsWith("tmp"));
+}
+
 TEST_F(RenameCollectionTest,
        RenameCollectionReturnsNamespaceExitsIfTargetExistsAndDropTargetIsFalse) {
     _createCollection(_opCtx.get(), _sourceNss);
@@ -522,6 +681,7 @@ TEST_F(RenameCollectionTest, RenameCollectionMakesTargetCollectionDropPendingIfD
                                                              << " missing after successful rename";
 
     ASSERT_TRUE(_opObserver->onRenameCollectionCalled);
+    ASSERT_TRUE(_opObserver->onRenameCollectionDropTarget);
 
     auto renameOpTime = _opObserver->renameOpTime;
     ASSERT_GREATER_THAN(renameOpTime, repl::OpTime());
@@ -573,6 +733,21 @@ TEST_F(RenameCollectionTest,
         << " renamed to drop-pending collection after successful rename";
 }
 
+TEST_F(RenameCollectionTest,
+       RenameCollectionOverridesDropTargetIfTargetCollectionIsMissingAndDropTargetIsTrue) {
+    _createCollectionWithUUID(_opCtx.get(), _sourceNss);
+    RenameCollectionOptions options;
+    options.dropTarget = true;
+    ASSERT_OK(renameCollection(_opCtx.get(), _sourceNss, _targetNss, options));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _sourceNss))
+        << "source collection " << _sourceNss << " still exists after successful rename";
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss)) << "target collection " << _targetNss
+                                                             << " missing after successful rename";
+
+    ASSERT_TRUE(_opObserver->onRenameCollectionCalled);
+    ASSERT_FALSE(_opObserver->onRenameCollectionDropTarget);
+}
+
 TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsRejectsRenameOpTimeIfWritesAreReplicated) {
     ASSERT_TRUE(_opCtx->writesAreReplicated());
 
@@ -624,6 +799,92 @@ DEATH_TEST_F(RenameCollectionTest,
     ASSERT_OK(renameCollectionForApplyOps(_opCtx.get(), dbName, {}, cmd, renameOpTime));
 }
 
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsSourceAndTargetDoNotExist) {
+    auto uuidDoc = BSON("ui" << UUID::gen());
+    auto cmd = BSON("renameCollection" << _sourceNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                       << "true");
+    ASSERT_EQUALS(ErrorCodes::NamespaceNotFound,
+                  renameCollectionForApplyOps(
+                      _opCtx.get(), _sourceNss.db().toString(), uuidDoc["ui"], cmd, {}));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _sourceNss));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _targetNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetEvenIfSourceDoesNotExist) {
+    _createCollectionWithUUID(_opCtx.get(), _targetNss);
+    auto missingSourceNss = NamespaceString("test.bar2");
+    auto uuidDoc = BSON("ui" << UUID::gen());
+    auto cmd =
+        BSON("renameCollection" << missingSourceNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                << "true");
+    ASSERT_OK(renameCollectionForApplyOps(
+        _opCtx.get(), missingSourceNss.db().toString(), uuidDoc["ui"], cmd, {}));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _targetNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetByUUIDEvenIfSourceDoesNotExist) {
+    auto missingSourceNss = NamespaceString("test.bar2");
+    auto dropTargetNss = NamespaceString("test.bar3");
+    _createCollectionWithUUID(_opCtx.get(), _targetNss);
+    auto dropTargetUUID = _createCollectionWithUUID(_opCtx.get(), dropTargetNss);
+    auto uuidDoc = BSON("ui" << UUID::gen());
+    auto cmd =
+        BSON("renameCollection" << missingSourceNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                << dropTargetUUID);
+    ASSERT_OK(renameCollectionForApplyOps(
+        _opCtx.get(), missingSourceNss.db().toString(), uuidDoc["ui"], cmd, {}));
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), dropTargetNss));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetEvenIfSourceIsDropPending) {
+    repl::OpTime dropOpTime(Timestamp(Seconds(100), 0), 1LL);
+    auto dropPendingNss = _sourceNss.makeDropPendingNamespace(dropOpTime);
+
+    auto dropTargetUUID = _createCollectionWithUUID(_opCtx.get(), _targetNss);
+    auto uuidDoc = BSON("ui" << _createCollectionWithUUID(_opCtx.get(), dropPendingNss));
+    auto cmd =
+        BSON("renameCollection" << dropPendingNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                << "true");
+
+    repl::UnreplicatedWritesBlock uwb(_opCtx.get());
+    repl::OpTime renameOpTime = {Timestamp(Seconds(200), 1U), 1LL};
+    ASSERT_OK(renameCollectionForApplyOps(
+        _opCtx.get(), dropPendingNss.db().toString(), uuidDoc["ui"], cmd, renameOpTime));
+
+    // Source collections stays in drop-pending state.
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), dropPendingNss));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), _targetNss));
+    ASSERT_EQUALS(_targetNss.makeDropPendingNamespace(renameOpTime),
+                  _getCollectionNssFromUUID(_opCtx.get(), dropTargetUUID));
+}
+
+TEST_F(RenameCollectionTest, RenameCollectionForApplyOpsDropTargetByUUIDEvenIfSourceIsDropPending) {
+    repl::OpTime dropOpTime(Timestamp(Seconds(100), 0), 1LL);
+    auto dropPendingNss = _sourceNss.makeDropPendingNamespace(dropOpTime);
+    auto dropTargetNss = NamespaceString("test.bar2");
+
+    _createCollectionWithUUID(_opCtx.get(), _targetNss);
+
+    auto dropTargetUUID = _createCollectionWithUUID(_opCtx.get(), dropTargetNss);
+    auto uuidDoc = BSON("ui" << _createCollectionWithUUID(_opCtx.get(), dropPendingNss));
+    auto cmd =
+        BSON("renameCollection" << dropPendingNss.ns() << "to" << _targetNss.ns() << "dropTarget"
+                                << dropTargetUUID);
+
+    repl::UnreplicatedWritesBlock uwb(_opCtx.get());
+    repl::OpTime renameOpTime = {Timestamp(Seconds(200), 1U), 1LL};
+    ASSERT_OK(renameCollectionForApplyOps(
+        _opCtx.get(), dropPendingNss.db().toString(), uuidDoc["ui"], cmd, renameOpTime));
+
+    // Source collections stays in drop-pending state.
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), dropPendingNss));
+    ASSERT_FALSE(_collectionExists(_opCtx.get(), dropTargetNss));
+    ASSERT_EQUALS(dropTargetNss.makeDropPendingNamespace(renameOpTime),
+                  _getCollectionNssFromUUID(_opCtx.get(), dropTargetUUID));
+    ASSERT_TRUE(_collectionExists(_opCtx.get(), _targetNss));
+}
+
 void _testRenameCollectionStayTemp(OperationContext* opCtx,
                                    const NamespaceString& sourceNss,
                                    const NamespaceString& targetNss,
