@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2012-2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -247,7 +249,16 @@ public:
 
         // Closing a database requires a global lock.
         Lock::GlobalWrite lk(opCtx);
-        if (!dbHolder().get(opCtx, dbname)) {
+        auto db = dbHolder().get(opCtx, dbname);
+        if (db) {
+            if (db->isDropPending(opCtx)) {
+                return appendCommandStatus(
+                    result,
+                    Status(ErrorCodes::DatabaseDropPending,
+                           str::stream() << "Cannot repair database " << dbname
+                                         << " since it is pending being dropped."));
+            }
+        } else {
             // If the name doesn't make an exact match, check for a case insensitive match.
             std::set<std::string> otherCasing = dbHolder().getNamesWithConflictingCasing(dbname);
             if (otherCasing.empty()) {
@@ -360,6 +371,12 @@ public:
         result.append("slowms", serverGlobalParams.slowMS);
         result.append("sampleRate", serverGlobalParams.sampleRate);
 
+        if (!serverGlobalParams.featureCompatibility.isVersionInitialized()) {
+            errmsg =
+                "profiling level cannot be set when featureCompatibilityVersion is uninitialized";
+            return false;
+        }
+
         if (!readOnly) {
             if (!db) {
                 // When setting the profiling level, create the database if it didn't already exist.
@@ -702,10 +719,8 @@ public:
 
             if (PlanExecutor::DEAD == state || PlanExecutor::FAILURE == state) {
                 return appendCommandStatus(result,
-                                           Status(ErrorCodes::OperationFailed,
-                                                  str::stream()
-                                                      << "Executor error during filemd5 command: "
-                                                      << WorkingSetCommon::toStatusString(obj)));
+                                           WorkingSetCommon::getMemberObjectStatus(obj).withContext(
+                                               "Executor error during filemd5 command"));
             }
 
             if (partialOk)
@@ -870,11 +885,9 @@ public:
 
         if (PlanExecutor::FAILURE == state || PlanExecutor::DEAD == state) {
             warning() << "Internal error while reading " << ns;
-            return appendCommandStatus(
-                result,
-                Status(ErrorCodes::OperationFailed,
-                       str::stream() << "Executor error while reading during dataSize command: "
-                                     << WorkingSetCommon::toStatusString(obj)));
+            return appendCommandStatus(result,
+                                       WorkingSetCommon::getMemberObjectStatus(obj).withContext(
+                                           "Executor error while reading during dataSize command"));
         }
 
         ostringstream os;
