@@ -1,29 +1,31 @@
+
 /**
- *    Copyright (C) 2012-2016 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
@@ -43,7 +45,6 @@
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/shutdown.h"
 #include "mongo/db/db.h"
-#include "mongo/db/instance.h"
 #include "mongo/db/introspect.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/json.h"
@@ -58,21 +59,22 @@
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
 #include "mongo/util/md5.hpp"
+#include "mongo/util/net/sock.h"
 #include "mongo/util/ntservice.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/ramlog.h"
 #include "mongo/util/version.h"
 
 namespace mongo {
+namespace {
 
-using std::endl;
 using std::string;
 using std::stringstream;
 using std::vector;
 
-class CmdBuildInfo : public Command {
+class CmdBuildInfo : public BasicCommand {
 public:
-    CmdBuildInfo() : Command("buildInfo", true, "buildinfo") {}
+    CmdBuildInfo() : BasicCommand("buildInfo", "buildinfo") {}
     virtual bool slaveOk() const {
         return true;
     }
@@ -90,11 +92,9 @@ public:
         help << "{ buildinfo:1 }";
     }
 
-    bool run(OperationContext* txn,
+    bool run(OperationContext* opCtx,
              const std::string& dbname,
-             BSONObj& jsobj,
-             int,  // options
-             std::string& errmsg,
+             const BSONObj& jsobj,
              BSONObjBuilder& result) {
         VersionInfoInterface::instance().appendBuildInfo(&result);
         appendStorageEngineList(&result);
@@ -103,10 +103,9 @@ public:
 
 } cmdBuildInfo;
 
-
-class PingCommand : public Command {
+class PingCommand : public BasicCommand {
 public:
-    PingCommand() : Command("ping") {}
+    PingCommand() : BasicCommand("ping") {}
     virtual bool slaveOk() const {
         return true;
     }
@@ -117,23 +116,27 @@ public:
     virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
     }
+    virtual bool allowsAfterClusterTime(const BSONObj& cmdObj) const override {
+        return false;
+    }
     virtual void addRequiredPrivileges(const std::string& dbname,
                                        const BSONObj& cmdObj,
                                        std::vector<Privilege>* out) {}  // No auth required
-    virtual bool run(OperationContext* txn,
+    virtual bool requiresAuth() const override {
+        return false;
+    }
+    virtual bool run(OperationContext* opCtx,
                      const string& badns,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         // IMPORTANT: Don't put anything in here that might lock db - including authentication
         return true;
     }
 } pingCmd;
 
-class FeaturesCmd : public Command {
+class FeaturesCmd : public BasicCommand {
 public:
-    FeaturesCmd() : Command("features", true) {}
+    FeaturesCmd() : BasicCommand("features") {}
     void help(stringstream& h) const {
         h << "return build level feature settings";
     }
@@ -146,11 +149,9 @@ public:
     virtual void addRequiredPrivileges(const std::string& dbname,
                                        const BSONObj& cmdObj,
                                        std::vector<Privilege>* out) {}  // No auth required
-    virtual bool run(OperationContext* txn,
+    virtual bool run(OperationContext* opCtx,
                      const string& ns,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         if (getGlobalScriptEngine()) {
             BSONObjBuilder bb(result.subobjStart("js"));
@@ -167,9 +168,9 @@ public:
 
 } featuresCmd;
 
-class HostInfoCmd : public Command {
+class HostInfoCmd : public BasicCommand {
 public:
-    HostInfoCmd() : Command("hostInfo", true) {}
+    HostInfoCmd() : BasicCommand("hostInfo") {}
     virtual bool slaveOk() const {
         return true;
     }
@@ -189,11 +190,9 @@ public:
         actions.addAction(ActionType::hostInfo);
         out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
     }
-    bool run(OperationContext* txn,
+    bool run(OperationContext* opCtx,
              const string& dbname,
-             BSONObj& cmdObj,
-             int,
-             string& errmsg,
+             const BSONObj& cmdObj,
              BSONObjBuilder& result) {
         ProcessInfo p;
         BSONObjBuilder bSys, bOs;
@@ -218,9 +217,9 @@ public:
 
 } hostInfoCmd;
 
-class LogRotateCmd : public Command {
+class LogRotateCmd : public BasicCommand {
 public:
-    LogRotateCmd() : Command("logRotate") {}
+    LogRotateCmd() : BasicCommand("logRotate") {}
     virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
     }
@@ -237,11 +236,9 @@ public:
         actions.addAction(ActionType::logRotate);
         out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
     }
-    virtual bool run(OperationContext* txn,
+    virtual bool run(OperationContext* opCtx,
                      const string& ns,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         bool didRotate = rotateLogs(serverGlobalParams.logRenameOnRotate);
         if (didRotate)
@@ -251,12 +248,12 @@ public:
 
 } logRotateCmd;
 
-class ListCommandsCmd : public Command {
+class ListCommandsCmd : public BasicCommand {
 public:
     virtual void help(stringstream& help) const {
         help << "get a list of all db commands";
     }
-    ListCommandsCmd() : Command("listCommands", false) {}
+    ListCommandsCmd() : BasicCommand("listCommands") {}
     virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
         return false;
     }
@@ -269,18 +266,16 @@ public:
     virtual void addRequiredPrivileges(const std::string& dbname,
                                        const BSONObj& cmdObj,
                                        std::vector<Privilege>* out) {}  // No auth required
-    virtual bool run(OperationContext* txn,
+    virtual bool run(OperationContext* opCtx,
                      const string& ns,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         // sort the commands before building the result BSON
         std::vector<Command*> commands;
-        for (CommandMap::const_iterator it = _commands->begin(); it != _commands->end(); ++it) {
+        for (const auto command : allCommands()) {
             // don't show oldnames
-            if (it->first == it->second->getName())
-                commands.push_back(it->second);
+            if (command.first == command.second->getName())
+                commands.push_back(command.second);
         }
         std::sort(commands.begin(), commands.end(), [](Command* lhs, Command* rhs) {
             return (lhs->getName()) < (rhs->getName());
@@ -309,51 +304,8 @@ public:
 
 } listCommandsCmd;
 
-namespace {
-MONGO_FP_DECLARE(crashOnShutdown);
-
-int* volatile illegalAddress;
-}  // namespace
-
-void CmdShutdown::addRequiredPrivileges(const std::string& dbname,
-                                        const BSONObj& cmdObj,
-                                        std::vector<Privilege>* out) {
-    ActionSet actions;
-    actions.addAction(ActionType::shutdown);
-    out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
-}
-
-void CmdShutdown::shutdownHelper() {
-    MONGO_FAIL_POINT_BLOCK(crashOnShutdown, crashBlock) {
-        const std::string crashHow = crashBlock.getData()["how"].str();
-        if (crashHow == "fault") {
-            ++*illegalAddress;
-        }
-        ::abort();
-    }
-
-    log() << "terminating, shutdown command received";
-
-#if defined(_WIN32)
-    // Signal the ServiceMain thread to shutdown.
-    if (ntservice::shouldStartService()) {
-        shutdownNoTerminate();
-
-        // Client expects us to abruptly close the socket as part of exiting
-        // so this function is not allowed to return.
-        // The ServiceMain thread will quit for us so just sleep until it does.
-        while (true)
-            sleepsecs(60);  // Loop forever
-    } else
-#endif
-    {
-        exitCleanly(EXIT_CLEAN);  // this never returns
-        invariant(false);
-    }
-}
-
 /* for testing purposes only */
-class CmdForceError : public Command {
+class CmdForceError : public BasicCommand {
 public:
     virtual void help(stringstream& help) const {
         help << "for testing purposes only.  forces a user assertion exception";
@@ -367,21 +319,19 @@ public:
     virtual void addRequiredPrivileges(const std::string& dbname,
                                        const BSONObj& cmdObj,
                                        std::vector<Privilege>* out) {}  // No auth required
-    CmdForceError() : Command("forceerror") {}
-    bool run(OperationContext* txn,
+    CmdForceError() : BasicCommand("forceerror") {}
+    bool run(OperationContext* opCtx,
              const string& dbnamne,
-             BSONObj& cmdObj,
-             int,
-             string& errmsg,
+             const BSONObj& cmdObj,
              BSONObjBuilder& result) {
         LastError::get(cc()).setLastError(10038, "forced error");
         return false;
     }
 } cmdForceError;
 
-class GetLogCmd : public Command {
+class GetLogCmd : public ErrmsgCommandDeprecated {
 public:
-    GetLogCmd() : Command("getLog") {}
+    GetLogCmd() : ErrmsgCommandDeprecated("getLog") {}
 
     virtual bool slaveOk() const {
         return true;
@@ -403,12 +353,11 @@ public:
         help << "{ getLog : '*' }  OR { getLog : 'global' }";
     }
 
-    virtual bool run(OperationContext* txn,
-                     const string& dbname,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
-                     BSONObjBuilder& result) {
+    virtual bool errmsgRun(OperationContext* opCtx,
+                           const string& dbname,
+                           const BSONObj& cmdObj,
+                           string& errmsg,
+                           BSONObjBuilder& result) {
         BSONElement val = cmdObj.firstElement();
         if (val.type() != String) {
             return appendCommandStatus(
@@ -451,9 +400,9 @@ public:
 
 } getLogCmd;
 
-class ClearLogCmd : public Command {
+class ClearLogCmd : public BasicCommand {
 public:
-    ClearLogCmd() : Command("clearLog") {}
+    ClearLogCmd() : BasicCommand("clearLog") {}
 
     virtual bool slaveOk() const {
         return true;
@@ -477,9 +426,7 @@ public:
 
     virtual bool run(OperationContext* opCtx,
                      const string& dbname,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         std::string logName;
         Status status = bsonExtractStringField(cmdObj, "clearLog", &logName);
@@ -506,9 +453,9 @@ MONGO_INITIALIZER(RegisterClearLogCmd)(InitializerContext* context) {
     return Status::OK();
 }
 
-class CmdGetCmdLineOpts : Command {
+class CmdGetCmdLineOpts : public BasicCommand {
 public:
-    CmdGetCmdLineOpts() : Command("getCmdLineOpts") {}
+    CmdGetCmdLineOpts() : BasicCommand("getCmdLineOpts") {}
     void help(stringstream& h) const {
         h << "get argv";
     }
@@ -528,11 +475,9 @@ public:
         actions.addAction(ActionType::getCmdLineOpts);
         out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
     }
-    virtual bool run(OperationContext* txn,
+    virtual bool run(OperationContext* opCtx,
                      const string&,
-                     BSONObj& cmdObj,
-                     int,
-                     string& errmsg,
+                     const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
         result.append("argv", serverGlobalParams.argvArray);
         result.append("parsed", serverGlobalParams.parsedOpts);
@@ -540,4 +485,47 @@ public:
     }
 
 } cmdGetCmdLineOpts;
+
+MONGO_FP_DECLARE(crashOnShutdown);
+int* volatile illegalAddress;  // NOLINT - used for fail point only
+
+}  // namespace
+
+void CmdShutdown::addRequiredPrivileges(const std::string& dbname,
+                                        const BSONObj& cmdObj,
+                                        std::vector<Privilege>* out) {
+    ActionSet actions;
+    actions.addAction(ActionType::shutdown);
+    out->push_back(Privilege(ResourcePattern::forClusterResource(), actions));
 }
+
+void CmdShutdown::shutdownHelper() {
+    MONGO_FAIL_POINT_BLOCK(crashOnShutdown, crashBlock) {
+        const std::string crashHow = crashBlock.getData()["how"].str();
+        if (crashHow == "fault") {
+            ++*illegalAddress;
+        }
+        ::abort();
+    }
+
+    log() << "terminating, shutdown command received";
+
+#if defined(_WIN32)
+    // Signal the ServiceMain thread to shutdown.
+    if (ntservice::shouldStartService()) {
+        shutdownNoTerminate();
+
+        // Client expects us to abruptly close the socket as part of exiting
+        // so this function is not allowed to return.
+        // The ServiceMain thread will quit for us so just sleep until it does.
+        while (true)
+            sleepsecs(60);  // Loop forever
+    } else
+#endif
+    {
+        exitCleanly(EXIT_CLEAN);  // this never returns
+        invariant(false);
+    }
+}
+
+}  // namespace mongo
