@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2012 10gen Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -369,11 +371,14 @@ void State::dropTempCollections() {
             AutoGetDb autoDb(_opCtx, _config.tempNamespace.db(), MODE_X);
             if (auto db = autoDb.getDb()) {
                 WriteUnitOfWork wunit(_opCtx);
-                uassert(ErrorCodes::PrimarySteppedDown,
-                        "no longer primary",
-                        repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(
-                            _opCtx, _config.tempNamespace));
-                db->dropCollection(_opCtx, _config.tempNamespace.ns()).transitional_ignore();
+                uassert(
+                    ErrorCodes::PrimarySteppedDown,
+                    str::stream()
+                        << "no longer primary while dropping temporary collection for mapReduce: "
+                        << _config.tempNamespace.ns(),
+                    repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(
+                        _opCtx, _config.tempNamespace));
+                uassertStatusOK(db->dropCollection(_opCtx, _config.tempNamespace.ns()));
                 wunit.commit();
             }
         });
@@ -389,7 +394,7 @@ void State::dropTempCollections() {
             Lock::DBLock lk(_opCtx, _config.incLong.db(), MODE_X);
             if (Database* db = dbHolder().get(_opCtx, _config.incLong.ns())) {
                 WriteUnitOfWork wunit(_opCtx);
-                db->dropCollection(_opCtx, _config.incLong.ns()).transitional_ignore();
+                uassertStatusOK(db->dropCollection(_opCtx, _config.incLong.ns()));
                 wunit.commit();
             }
         });
@@ -457,20 +462,6 @@ void State::prepTempCollection() {
         if (finalColl) {
             finalOptions = finalColl->getCatalogEntry()->getCollectionOptions(_opCtx);
 
-            if (_config.finalOutputCollUUID) {
-                // The final output collection's UUID is passed from mongos if the final output
-                // collection is sharded. If a UUID was sent, ensure it matches what's on this
-                // shard.
-                uassert(ErrorCodes::InternalError,
-                        str::stream()
-                            << "UUID sent by mongos for sharded final output collection "
-                            << _config.outputOptions.finalNamespace.ns()
-                            << " does not match UUID for the existing collection with that "
-                               "name on this shard",
-                        finalColl->getCatalogEntry()->isEqualToMetadataUUID(
-                            _opCtx, _config.finalOutputCollUUID));
-            }
-
             IndexCatalog::IndexIterator ii =
                 finalColl->getIndexCatalog()->getIndexIterator(_opCtx, true);
             // Iterate over finalColl's indexes.
@@ -496,10 +487,12 @@ void State::prepTempCollection() {
         // create temp collection and insert the indexes from temporary storage
         OldClientWriteContext tempCtx(_opCtx, _config.tempNamespace.ns());
         WriteUnitOfWork wuow(_opCtx);
-        uassert(ErrorCodes::PrimarySteppedDown,
-                "no longer primary",
-                repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(_opCtx,
-                                                                            _config.tempNamespace));
+        uassert(
+            ErrorCodes::PrimarySteppedDown,
+            str::stream() << "no longer primary while creating temporary collection for mapReduce: "
+                          << _config.tempNamespace.ns(),
+            repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(_opCtx,
+                                                                        _config.tempNamespace));
         Collection* tempColl = tempCtx.getCollection();
         invariant(!tempColl);
 
@@ -754,9 +747,13 @@ void State::insert(const NamespaceString& nss, const BSONObj& o) {
     writeConflictRetry(_opCtx, "M/R insert", nss.ns(), [this, &nss, &o] {
         OldClientWriteContext ctx(_opCtx, nss.ns());
         WriteUnitOfWork wuow(_opCtx);
-        uassert(ErrorCodes::PrimarySteppedDown,
-                "no longer primary",
-                repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(_opCtx, nss));
+        uassert(
+            ErrorCodes::PrimarySteppedDown,
+            str::stream() << "no longer primary while inserting mapReduce result into collection: "
+                          << nss.ns()
+                          << ": "
+                          << redact(o),
+            repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(_opCtx, nss));
         Collection* coll = getCollectionOrUassert(_opCtx, ctx.db(), nss);
 
         BSONObjBuilder b;
@@ -831,8 +828,11 @@ State::~State() {
     if (_onDisk) {
         try {
             dropTempCollections();
-        } catch (std::exception& e) {
-            error() << "couldn't cleanup after map reduce: " << e.what();
+        } catch (...) {
+            error() << "Unable to drop temporary collection created by mapReduce: "
+                    << _config.tempNamespace << ". This collection will be removed automatically "
+                                                "the next time the server starts up. "
+                    << exceptionToStatus();
         }
     }
     if (_scope && !_scope->isKillPending() && _scope->getError().empty()) {
@@ -1420,18 +1420,17 @@ public:
 
         BSONObjBuilder countsBuilder;
         BSONObjBuilder timingBuilder;
-        State state(opCtx, config);
-        if (!state.sourceExists()) {
-            return appendCommandStatus(
-                result,
-                Status(ErrorCodes::NamespaceNotFound,
-                       str::stream() << "namespace does not exist: " << config.nss.ns()));
-        }
-
         try {
+            State state(opCtx, config);
+            if (!state.sourceExists()) {
+                return appendCommandStatus(
+                    result,
+                    Status(ErrorCodes::NamespaceNotFound,
+                           str::stream() << "namespace does not exist: " << config.nss.ns()));
+            }
+
             state.init();
             state.prepTempCollection();
-            ON_BLOCK_EXIT_OBJ(state, &State::dropTempCollections);
 
             int progressTotal = 0;
             bool showTotal = true;
@@ -1783,7 +1782,6 @@ public:
         }
 
         state.prepTempCollection();
-        ON_BLOCK_EXIT_OBJ(state, &State::dropTempCollections);
 
         std::vector<std::shared_ptr<Chunk>> chunks;
 
