@@ -1,32 +1,36 @@
 // parameters.cpp
 
+
 /**
-*    Copyright (C) 2012 10gen Inc.
-*
-*    This program is free software: you can redistribute it and/or  modify
-*    it under the terms of the GNU Affero General Public License, version 3,
-*    as published by the Free Software Foundation.
-*
-*    This program is distributed in the hope that it will be useful,
-*    but WITHOUT ANY WARRANTY; without even the implied warranty of
-*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*    GNU Affero General Public License for more details.
-*
-*    You should have received a copy of the GNU Affero General Public License
-*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*
-*    As a special exception, the copyright holders give permission to link the
-*    code of portions of this program with the OpenSSL library under certain
-*    conditions as described in each individual source file and distribute
-*    linked combinations including the program with the OpenSSL library. You
-*    must comply with the GNU Affero General Public License in all respects for
-*    all of the code used other than as permitted herein. If you modify file(s)
-*    with this exception, you may extend this exception to your version of the
-*    file(s), but you are not obligated to do so. If you do not wish to do so,
-*    delete this exception statement from your version. If you delete this
-*    exception statement from all source files in the program, then also delete
-*    it in the license file.
-*/
+ *    Copyright (C) 2018-present MongoDB, Inc.
+ *
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
+ *
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    Server Side Public License for more details.
+ *
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
+ *
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
+ */
+
+#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -44,6 +48,7 @@
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/logger/logger.h"
 #include "mongo/logger/parse_log_component_settings.h"
+#include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
@@ -212,11 +217,34 @@ public:
                 return false;
             }
 
-            if (numSet == 0) {
-                foundParameter->second->append(opCtx, result, "was");
+            auto oldValueObj = ([&] {
+                BSONObjBuilder bb;
+                if (numSet == 0) {
+                    foundParameter->second->append(opCtx, bb, "was");
+                }
+                return bb.obj();
+            })();
+            auto oldValue = oldValueObj.firstElement();
+
+            if (oldValue) {
+                result.append(oldValue);
             }
 
-            uassertStatusOK(foundParameter->second->set(parameter));
+            try {
+                uassertStatusOK(foundParameter->second->set(parameter));
+            } catch (const DBException& ex) {
+                log() << "error setting parameter " << parameterName << " to "
+                      << redact(parameter.toString(false)) << " errMsg: " << redact(ex);
+                throw;
+            }
+
+            log() << "successfully set parameter " << parameterName << " to "
+                  << redact(parameter.toString(false))
+                  << (oldValue ? std::string(str::stream() << " (was "
+                                                           << redact(oldValue.toString(false))
+                                                           << ")")
+                               : "");
+
             numSet++;
         }
 
@@ -570,7 +598,7 @@ public:
                      << saslCommandUserDBFieldName
                      << "$external"
                      << saslCommandUserFieldName
-                     << getSSLManager()->getSSLConfiguration().clientSubjectName));
+                     << getSSLManager()->getSSLConfiguration().clientSubjectName.toString()));
 #endif
         } else if (str == "x509" && oldMode == ServerGlobalParams::ClusterAuthMode_sendX509) {
             serverGlobalParams.clusterAuthMode.store(ServerGlobalParams::ClusterAuthMode_x509);
