@@ -1,29 +1,31 @@
+
 /**
- * Copyright (c) 2012-2014 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- * This program is free software: you can redistribute it and/or  modify
- * it under the terms of the GNU Affero General Public License, version 3,
- * as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    Server Side Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
- * As a special exception, the copyright holders give permission to link the
- * code of portions of this program with the OpenSSL library under certain
- * conditions as described in each individual source file and distribute
- * linked combinations including the program with the OpenSSL library. You
- * must comply with the GNU Affero General Public License in all respects for
- * all of the code used other than as permitted herein. If you modify file(s)
- * with this exception, you may extend this exception to your version of the
- * file(s), but you are not obligated to do so. If you do not wish to do so,
- * delete this exception statement from your version. If you delete this
- * exception statement from all source files in the program, then also delete
- * it in the license file.
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
@@ -334,7 +336,8 @@ public:
                                   (truncateMode == CurrentOpTruncateMode::kTruncateOps));
 
                 Locker::LockerInfo lockerInfo;
-                clientOpCtx->lockState()->getLockerInfo(&lockerInfo);
+                clientOpCtx->lockState()->getLockerInfo(
+                    &lockerInfo, CurOp::get(clientOpCtx)->getLockStatsBase());
                 fillLockerInfo(lockerInfo, infoBuilder);
             }
 
@@ -625,7 +628,13 @@ void PipelineD::prepareCursorSource(Collection* collection,
                     pipeline,
                     expCtx,
                     std::move(exec),
-                    pipeline->getDependencies(DepsTracker::MetadataAvailable::kNoMetadata));
+                    pipeline->getDependencies(DepsTracker::MetadataAvailable::kNoMetadata),
+                    BSONObj(),  // queryObj
+                    BSONObj(),  // sortObj
+                    BSONObj(),  // projectionObj
+                    // Do not allow execution-level explains on this DocumentSourceCursor,
+                    // as it contains a RandomCursor, which will never return EOF.
+                    true);
                 return;
             }
         }
@@ -870,13 +879,14 @@ void PipelineD::addCursorSource(Collection* collection,
                                 DepsTracker deps,
                                 const BSONObj& queryObj,
                                 const BSONObj& sortObj,
-                                const BSONObj& projectionObj) {
+                                const BSONObj& projectionObj,
+                                bool failsForExecutionLevelExplain) {
     // DocumentSourceCursor expects a yielding PlanExecutor that has had its state saved.
     exec->saveState();
 
     // Put the PlanExecutor into a DocumentSourceCursor and add it to the front of the pipeline.
-    intrusive_ptr<DocumentSourceCursor> pSource =
-        DocumentSourceCursor::create(collection, std::move(exec), expCtx);
+    intrusive_ptr<DocumentSourceCursor> pSource = DocumentSourceCursor::create(
+        collection, std::move(exec), expCtx, failsForExecutionLevelExplain);
 
     // Note the query, sort, and projection for explain.
     pSource->setQuery(queryObj);
