@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2012 10gen Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -79,6 +81,22 @@ const auto defaultBatchSize = (16 * 1024 * 1024) / 12 * 10;
 // The batchSize to use for the find/getMore queries called by the OplogFetcher
 MONGO_EXPORT_STARTUP_SERVER_PARAMETER(bgSyncOplogFetcherBatchSize, int, defaultBatchSize);
 
+// The batchSize to use for the find/getMore queries called by the rollback common point resolver.
+// A batchSize of 0 means that the 'find' and 'getMore' commands will be given no batchSize.
+// We set the default to 2000 to prevent the sync source from having to read too much data at once,
+// and reduce the chance of a socket timeout.
+// We choose 2000 for (10 minute timeout) * (60 sec / min) * (50 MB / second) / (16 MB / document).
+constexpr int defaultRollbackBatchSize = 2000;
+MONGO_EXPORT_SERVER_PARAMETER(rollbackRemoteOplogQueryBatchSize, int, defaultRollbackBatchSize)
+    ->withValidator([](const auto& potentialNewValue) {
+        if (potentialNewValue < 0) {
+            return Status(ErrorCodes::BadValue,
+                          "rollbackRemoteOplogQueryBatchSize cannot be negative.");
+        }
+
+        return Status::OK();
+    });
+
 /**
  * Extends DataReplicatorExternalStateImpl to be member state aware.
  */
@@ -295,6 +313,9 @@ void BackgroundSync::_produce() {
         }
         const auto requiredOpTime = (minValidSaved > _lastOpTimeFetched) ? minValidSaved : OpTime();
         lastOpTimeFetched = _lastOpTimeFetched;
+        if (!_syncSourceHost.empty()) {
+            log() << "Clearing sync source " << _syncSourceHost << " to choose a new one.";
+        }
         _syncSourceHost = HostAndPort();
         _syncSourceResolver = stdx::make_unique<SyncSourceResolver>(
             _replicationCoordinatorExternalState->getTaskExecutor(),
@@ -373,6 +394,10 @@ void BackgroundSync::_produce() {
                   << ". Sleeping for 1 second to avoid immediately choosing a new sync source for "
                      "the same reason as last time.";
             sleepsecs(1);
+        } else {
+            log() << "Changed sync source from "
+                  << (oldSource.empty() ? std::string("empty") : oldSource.toString()) << " to "
+                  << source;
         }
     } else {
         if (!syncSourceResp.isOK()) {
@@ -390,7 +415,7 @@ void BackgroundSync::_produce() {
 
         _tooStale = false;
 
-        log() << "No longer too stale. Able to sync from " << _syncSourceHost;
+        log() << "No longer too stale. Able to sync from " << source;
 
         auto status = _replCoord->setMaintenanceMode(false);
         if (!status.isOK()) {
@@ -440,7 +465,7 @@ void BackgroundSync::_produce() {
             source,
             NamespaceString::kRsOplogNamespace,
             _replCoord->getConfig(),
-            _replicationCoordinatorExternalState->getOplogFetcherMaxFetcherRestarts(),
+            _replicationCoordinatorExternalState->getOplogFetcherSteadyStateMaxFetcherRestarts(),
             syncSourceResp.rbid,
             true /* requireFresherSyncSource */,
             &dataReplicatorExternalState,
@@ -462,8 +487,8 @@ void BackgroundSync::_produce() {
     }
 
     const auto logLevel = Command::testCommandsEnabled ? 0 : 1;
-    LOG(logLevel) << "scheduling fetcher to read remote oplog on " << _syncSourceHost
-                  << " starting at " << oplogFetcher->getFindQuery_forTest()["filter"];
+    LOG(logLevel) << "scheduling fetcher to read remote oplog on " << source << " starting at "
+                  << oplogFetcher->getFindQuery_forTest()["filter"];
     auto scheduleStatus = oplogFetcher->startup();
     if (!scheduleStatus.isOK()) {
         warning() << "unable to schedule fetcher to read remote oplog on " << source << ": "
@@ -678,7 +703,9 @@ void BackgroundSync::_runRollbackViaRecoverToCheckpoint(
     StorageInterface* storageInterface,
     OplogInterfaceRemote::GetConnectionFn getConnection) {
 
-    OplogInterfaceRemote remoteOplog(getConnection, NamespaceString::kRsOplogNamespace.ns());
+    OplogInterfaceRemote remoteOplog(getConnection,
+                                     NamespaceString::kRsOplogNamespace.ns(),
+                                     rollbackRemoteOplogQueryBatchSize.load());
 
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
@@ -709,8 +736,10 @@ void BackgroundSync::_fallBackOnRollbackViaRefetch(
     bool useUUID,
     OplogInterfaceRemote::GetConnectionFn getConnection) {
 
-    RollbackSourceImpl rollbackSource(
-        getConnection, source, NamespaceString::kRsOplogNamespace.ns());
+    RollbackSourceImpl rollbackSource(getConnection,
+                                      source,
+                                      NamespaceString::kRsOplogNamespace.ns(),
+                                      rollbackRemoteOplogQueryBatchSize.load());
 
     if (useUUID) {
         rollback(opCtx, *localOplog, rollbackSource, requiredRBID, _replCoord, _replicationProcess);
@@ -727,6 +756,7 @@ HostAndPort BackgroundSync::getSyncTarget() const {
 
 void BackgroundSync::clearSyncTarget() {
     stdx::unique_lock<stdx::mutex> lock(_mutex);
+    log() << "Resetting sync source to empty, which was " << _syncSourceHost;
     _syncSourceHost = HostAndPort();
 }
 
