@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2014 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -31,13 +33,11 @@
 #include "mongo/base/status.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/repl/elect_cmd_runner.h"
-#include "mongo/db/repl/member_heartbeat_data.h"
 #include "mongo/db/repl/repl_set_config.h"
-#include "mongo/db/repl/replication_executor.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/executor/thread_pool_task_executor_test_fixture.h"
 #include "mongo/stdx/functional.h"
 #include "mongo/stdx/memory.h"
-#include "mongo/stdx/thread.h"
 #include "mongo/unittest/unittest.h"
 
 
@@ -51,7 +51,7 @@ using executor::NetworkInterfaceMock;
 using executor::RemoteCommandRequest;
 using executor::RemoteCommandResponse;
 
-class ElectCmdRunnerTest : public mongo::unittest::Test {
+class ElectCmdRunnerTest : public executor::ThreadPoolExecutorTest {
 public:
     void startTest(ElectCmdRunner* electCmdRunner,
                    const ReplSetConfig& currentConfig,
@@ -60,35 +60,21 @@ public:
 
     void waitForTest();
 
-    void electCmdRunnerRunner(const ReplicationExecutor::CallbackArgs& data,
+    void electCmdRunnerRunner(const executor::TaskExecutor::CallbackArgs& data,
                               ElectCmdRunner* electCmdRunner,
-                              StatusWith<ReplicationExecutor::EventHandle>* evh,
+                              StatusWith<executor::TaskExecutor::EventHandle>* evh,
                               const ReplSetConfig& currentConfig,
                               int selfIndex,
                               const std::vector<HostAndPort>& hosts);
 
-    NetworkInterfaceMock* _net;
-    std::unique_ptr<ReplicationExecutor> _executor;
-    std::unique_ptr<stdx::thread> _executorThread;
-
 private:
-    void setUp();
-    void tearDown();
-
-    ReplicationExecutor::EventHandle _allDoneEvent;
+    void setUp() {
+        executor::ThreadPoolExecutorTest::setUp();
+        launchExecutorThread();
+    }
+    executor::TaskExecutor::EventHandle _allDoneEvent;
 };
 
-void ElectCmdRunnerTest::setUp() {
-    _net = new NetworkInterfaceMock;
-    _executor = stdx::make_unique<ReplicationExecutor>(_net, 1 /* prng seed */);
-    _executorThread.reset(new stdx::thread(stdx::bind(&ReplicationExecutor::run, _executor.get())));
-}
-
-void ElectCmdRunnerTest::tearDown() {
-    _executor->shutdown();
-    _executorThread->join();
-}
-
 ReplSetConfig assertMakeRSConfig(const BSONObj& configBson) {
     ReplSetConfig config;
     ASSERT_OK(config.initialize(configBson));
@@ -122,40 +108,38 @@ BSONObj stripRound(const BSONObj& orig) {
 
 // This is necessary because the run method must be scheduled in the Replication Executor
 // for correct concurrency operation.
-void ElectCmdRunnerTest::electCmdRunnerRunner(const ReplicationExecutor::CallbackArgs& data,
+void ElectCmdRunnerTest::electCmdRunnerRunner(const executor::TaskExecutor::CallbackArgs& data,
                                               ElectCmdRunner* electCmdRunner,
-                                              StatusWith<ReplicationExecutor::EventHandle>* evh,
+                                              StatusWith<executor::TaskExecutor::EventHandle>* evh,
                                               const ReplSetConfig& currentConfig,
                                               int selfIndex,
                                               const std::vector<HostAndPort>& hosts) {
     invariant(data.status.isOK());
-    ReplicationExecutor* executor = dynamic_cast<ReplicationExecutor*>(data.executor);
-    ASSERT(executor);
-    *evh = electCmdRunner->start(executor, currentConfig, selfIndex, hosts);
+    *evh = electCmdRunner->start(data.executor, currentConfig, selfIndex, hosts);
 }
 
 void ElectCmdRunnerTest::startTest(ElectCmdRunner* electCmdRunner,
                                    const ReplSetConfig& currentConfig,
                                    int selfIndex,
                                    const std::vector<HostAndPort>& hosts) {
-    StatusWith<ReplicationExecutor::EventHandle> evh(ErrorCodes::InternalError, "Not set");
-    StatusWith<ReplicationExecutor::CallbackHandle> cbh =
-        _executor->scheduleWork(stdx::bind(&ElectCmdRunnerTest::electCmdRunnerRunner,
-                                           this,
-                                           stdx::placeholders::_1,
-                                           electCmdRunner,
-                                           &evh,
-                                           currentConfig,
-                                           selfIndex,
-                                           hosts));
+    StatusWith<executor::TaskExecutor::EventHandle> evh(ErrorCodes::InternalError, "Not set");
+    StatusWith<executor::TaskExecutor::CallbackHandle> cbh =
+        getExecutor().scheduleWork(stdx::bind(&ElectCmdRunnerTest::electCmdRunnerRunner,
+                                              this,
+                                              stdx::placeholders::_1,
+                                              electCmdRunner,
+                                              &evh,
+                                              currentConfig,
+                                              selfIndex,
+                                              hosts));
     ASSERT_OK(cbh.getStatus());
-    _executor->wait(cbh.getValue());
+    getExecutor().wait(cbh.getValue());
     ASSERT_OK(evh.getStatus());
     _allDoneEvent = evh.getValue();
 }
 
 void ElectCmdRunnerTest::waitForTest() {
-    _executor->waitForEvent(_allDoneEvent);
+    getExecutor().waitForEvent(_allDoneEvent);
 }
 
 TEST_F(ElectCmdRunnerTest, OneNode) {
@@ -194,21 +178,21 @@ TEST_F(ElectCmdRunnerTest, TwoNodes) {
 
     ElectCmdRunner electCmdRunner;
     startTest(&electCmdRunner, config, 0, hosts);
-    const Date_t startDate = _net->now();
-    _net->enterNetwork();
-    const NetworkInterfaceMock::NetworkOperationIterator noi = _net->getNextReadyRequest();
+    const Date_t startDate = getNet()->now();
+    getNet()->enterNetwork();
+    const NetworkInterfaceMock::NetworkOperationIterator noi = getNet()->getNextReadyRequest();
     ASSERT_EQUALS("admin", noi->getRequest().dbname);
     ASSERT_BSONOBJ_EQ(stripRound(electRequest), stripRound(noi->getRequest().cmdObj));
     ASSERT_EQUALS(HostAndPort("h1"), noi->getRequest().target);
-    _net->scheduleResponse(noi,
-                           startDate + Milliseconds(10),
-                           ResponseStatus(RemoteCommandResponse(
-                               BSON("ok" << 1 << "vote" << 1 << "round" << 380865962699346850ll),
+    getNet()->scheduleResponse(
+        noi,
+        startDate + Milliseconds(10),
+        (RemoteCommandResponse(BSON("ok" << 1 << "vote" << 1 << "round" << 380865962699346850ll),
                                BSONObj(),
                                Milliseconds(8))));
-    _net->runUntil(startDate + Milliseconds(10));
-    _net->exitNetwork();
-    ASSERT_EQUALS(startDate + Milliseconds(10), _net->now());
+    getNet()->runUntil(startDate + Milliseconds(10));
+    getNet()->exitNetwork();
+    ASSERT_EQUALS(startDate + Milliseconds(10), getNet()->now());
     waitForTest();
     ASSERT_EQUALS(electCmdRunner.getReceivedVotes(), 2);
 }
@@ -229,21 +213,22 @@ TEST_F(ElectCmdRunnerTest, ShuttingDown) {
     hosts.push_back(config.getMemberAt(1).getHostAndPort());
 
     ElectCmdRunner electCmdRunner;
-    StatusWith<ReplicationExecutor::EventHandle> evh(ErrorCodes::InternalError, "Not set");
-    StatusWith<ReplicationExecutor::CallbackHandle> cbh =
-        _executor->scheduleWork(stdx::bind(&ElectCmdRunnerTest::electCmdRunnerRunner,
-                                           this,
-                                           stdx::placeholders::_1,
-                                           &electCmdRunner,
-                                           &evh,
-                                           config,
-                                           0,
-                                           hosts));
+    StatusWith<executor::TaskExecutor::EventHandle> evh(ErrorCodes::InternalError, "Not set");
+    StatusWith<executor::TaskExecutor::CallbackHandle> cbh =
+        getExecutor().scheduleWork(stdx::bind(&ElectCmdRunnerTest::electCmdRunnerRunner,
+                                              this,
+                                              stdx::placeholders::_1,
+                                              &electCmdRunner,
+                                              &evh,
+                                              config,
+                                              0,
+                                              hosts));
     ASSERT_OK(cbh.getStatus());
-    _executor->wait(cbh.getValue());
+    getExecutor().wait(cbh.getValue());
     ASSERT_OK(evh.getStatus());
-    _executor->shutdown();
-    _executor->waitForEvent(evh.getValue());
+    shutdownExecutorThread();
+    joinExecutorThread();
+    getExecutor().waitForEvent(evh.getValue());
     ASSERT_EQUALS(electCmdRunner.getReceivedVotes(), 1);
 }
 
@@ -253,7 +238,7 @@ public:
         int selfConfigIndex = 0;
 
         ReplSetConfig config;
-        config.initialize(configObj);
+        config.initialize(configObj).transitional_ignore();
 
         std::vector<HostAndPort> hosts;
         for (ReplSetConfig::MemberIterator mem = ++config.membersBegin();
@@ -278,7 +263,8 @@ protected:
         return _checker->getReceivedVotes();
     }
 
-    void processResponse(const RemoteCommandRequest& request, const ResponseStatus& response) {
+    void processResponse(const RemoteCommandRequest& request,
+                         const RemoteCommandResponse& response) {
         _checker->processResponse(request, response);
     }
 
@@ -290,27 +276,27 @@ protected:
                                     Milliseconds(0));
     }
 
-    ResponseStatus badResponseStatus() {
-        return ResponseStatus(ErrorCodes::NodeNotFound, "not on my watch");
+    RemoteCommandResponse badRemoteCommandResponse() {
+        return RemoteCommandResponse(ErrorCodes::NodeNotFound, "not on my watch");
     }
 
-    ResponseStatus wrongTypeForVoteField() {
-        return ResponseStatus(NetworkInterfaceMock::Response(
+    RemoteCommandResponse wrongTypeForVoteField() {
+        return RemoteCommandResponse(NetworkInterfaceMock::Response(
             BSON("vote" << std::string("yea")), BSONObj(), Milliseconds(10)));
     }
 
-    ResponseStatus voteYea() {
-        return ResponseStatus(
+    RemoteCommandResponse voteYea() {
+        return RemoteCommandResponse(
             NetworkInterfaceMock::Response(BSON("vote" << 1), BSONObj(), Milliseconds(10)));
     }
 
-    ResponseStatus voteNay() {
-        return ResponseStatus(
+    RemoteCommandResponse voteNay() {
+        return RemoteCommandResponse(
             NetworkInterfaceMock::Response(BSON("vote" << -10000), BSONObj(), Milliseconds(10)));
     }
 
-    ResponseStatus abstainFromVoting() {
-        return ResponseStatus(
+    RemoteCommandResponse abstainFromVoting() {
+        return RemoteCommandResponse(
             NetworkInterfaceMock::Response(BSON("vote" << 0), BSONObj(), Milliseconds(10)));
     }
 
@@ -363,7 +349,7 @@ TEST_F(ElectScatterGatherTest, NodeRespondsWithBadStatus) {
     start(basicThreeNodeConfig());
     ASSERT_FALSE(hasReceivedSufficientResponses());
 
-    processResponse(requestFrom("host2"), badResponseStatus());
+    processResponse(requestFrom("host2"), badRemoteCommandResponse());
     ASSERT_FALSE(hasReceivedSufficientResponses());
 
     processResponse(requestFrom("host3"), abstainFromVoting());
@@ -405,7 +391,7 @@ TEST_F(ElectScatterGatherTest, NodeRespondsWithBadStatusArbiters) {
     start(threeNodesTwoArbitersConfig());
     ASSERT_FALSE(hasReceivedSufficientResponses());
 
-    processResponse(requestFrom("host2"), badResponseStatus());
+    processResponse(requestFrom("host2"), badRemoteCommandResponse());
     ASSERT_FALSE(hasReceivedSufficientResponses());
 
     processResponse(requestFrom("host3"), abstainFromVoting());
