@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -47,6 +49,7 @@
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/rollback_checker.h"
 #include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/repl/sync_tail.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/functional.h"
 #include "mongo/stdx/mutex.h"
@@ -56,17 +59,6 @@
 namespace mongo {
 namespace repl {
 
-namespace {
-
-using CallbackArgs = executor::TaskExecutor::CallbackArgs;
-using Event = executor::TaskExecutor::EventHandle;
-using Handle = executor::TaskExecutor::CallbackHandle;
-using Operations = MultiApplier::Operations;
-using QueryResponseStatus = StatusWith<Fetcher::QueryResponse>;
-using UniqueLock = stdx::unique_lock<stdx::mutex>;
-
-}  // namespace
-
 // TODO: Remove forward declares once we remove rs_initialsync.cpp and other dependents.
 // Failpoint which fails initial sync and leaves an oplog entry in the buffer.
 MONGO_FP_FORWARD_DECLARE(failInitSyncWithBufferedEntriesLeft);
@@ -83,15 +75,16 @@ MONGO_FP_FORWARD_DECLARE(rsSyncApplyStop);
 
 struct InitialSyncState;
 struct MemberState;
+class ReplicationProcess;
 class StorageInterface;
 
-
 struct InitialSyncerOptions {
     /** Function to return optime of last operation applied on this node */
     using GetMyLastOptimeFn = stdx::function<OpTime()>;
 
     /** Function to update optime of last operation applied on this node */
-    using SetMyLastOptimeFn = stdx::function<void(const OpTime&)>;
+    using SetMyLastOptimeFn =
+        stdx::function<void(const OpTime&, ReplicationCoordinator::DataConsistency consistency)>;
 
     /** Function to reset all optimes on this node (e.g. applied & durable). */
     using ResetOptimesFn = stdx::function<void()>;
@@ -102,6 +95,11 @@ struct InitialSyncerOptions {
     /** Function to get this node's slaveDelay. */
     using GetSlaveDelayFn = stdx::function<Seconds()>;
 
+    /**
+     * Struct to hold oplog application batch limits.
+     */
+    using BatchLimits = SyncTail::BatchLimits;
+
     // Error and retry values
     Milliseconds syncSourceRetryWait{1000};
     Milliseconds initialSyncRetryWait{1000};
@@ -115,8 +113,7 @@ struct InitialSyncerOptions {
     Milliseconds getApplierBatchCallbackRetryWait{1000};
 
     // Batching settings.
-    std::uint32_t replBatchLimitBytes = 512 * 1024 * 1024;
-    std::uint32_t replBatchLimitOperations = 5000;
+    BatchLimits batchLimits;
 
     // Replication settings
     NamespaceString localOplogNS = NamespaceString("local.oplog.rs");
@@ -190,6 +187,7 @@ public:
     InitialSyncer(InitialSyncerOptions opts,
                   std::unique_ptr<DataReplicatorExternalState> dataReplicatorExternalState,
                   StorageInterface* storage,
+                  ReplicationProcess* replicationProcess,
                   const OnCompletionFn& onCompletion);
 
     virtual ~InitialSyncer();
@@ -202,7 +200,7 @@ public:
     /**
      * Starts initial sync process, with the provided number of attempts
      */
-    Status startup(OperationContext* txn, std::uint32_t maxAttempts) noexcept;
+    Status startup(OperationContext* opCtx, std::uint32_t maxAttempts) noexcept;
 
     /**
      * Shuts down replication if "start" has been called, and blocks until shutdown has completed.
@@ -289,7 +287,7 @@ private:
      *         |
      *         |
      *         V
-     *    _recreateOplogAndDropReplicatedDatabases()
+     *    _truncateOplogAndDropReplicatedDatabases()
      *         |
      *         |
      *         V
@@ -300,6 +298,10 @@ private:
      *    _lastOplogEntryFetcherCallbackForBeginTimestamp()
      *         |
      *         |
+     *         V
+     *    _fcvFetcherCallback()
+     *         |
+     *         |
      *         +------------------------------+
      *         |                              |
      *         |                              |
@@ -352,12 +354,12 @@ private:
     /**
      * Sets up internal state to begin initial sync.
      */
-    void _setUp_inlock(OperationContext* txn, std::uint32_t initialSyncMaxAttempts);
+    void _setUp_inlock(OperationContext* opCtx, std::uint32_t initialSyncMaxAttempts);
 
     /**
      * Tears down internal state before reporting final status to caller.
      */
-    void _tearDown_inlock(OperationContext* txn, const StatusWith<OpTimeWithHash>& lastApplied);
+    void _tearDown_inlock(OperationContext* opCtx, const StatusWith<OpTimeWithHash>& lastApplied);
 
     /**
      * Callback to start a single initial sync attempt.
@@ -379,11 +381,10 @@ private:
 
     /**
      * This function does the following:
-     *      1.) Drop oplog.
+     *      1.) Truncate oplog.
      *      2.) Drop user databases (replicated dbs).
-     *      3.) Create oplog.
      */
-    Status _recreateOplogAndDropReplicatedDatabases();
+    Status _truncateOplogAndDropReplicatedDatabases();
 
     /**
      * Callback for rollback checker's first replSetGetRBID command before starting data cloning.
@@ -401,11 +402,19 @@ private:
         const StatusWith<Fetcher::QueryResponse>& result,
         std::shared_ptr<OnCompletionGuard> onCompletionGuard);
 
+
+    /**
+     * Callback for the '_fCVFetcher'. A successful response lets us check if the remote node
+     * is in a currently acceptable fCV and if it has a 'targetVersion' set.
+     */
+    void _fcvFetcherCallback(const StatusWith<Fetcher::QueryResponse>& result,
+                             std::shared_ptr<OnCompletionGuard> onCompletionGuard,
+                             const OpTimeWithHash& lastOpTimeWithHash);
+
     /**
      * Callback for oplog fetcher.
      */
     void _oplogFetcherCallback(const Status& status,
-                               const OpTimeWithHash& lastFetched,
                                std::shared_ptr<OnCompletionGuard> onCompletionGuard);
 
     /**
@@ -484,7 +493,7 @@ private:
     void _appendInitialSyncProgressMinimal_inlock(BSONObjBuilder* bob) const;
     BSONObj _getInitialSyncProgress_inlock() const;
 
-    StatusWith<Operations> _getNextApplierBatch_inlock();
+    StatusWith<MultiApplier::Operations> _getNextApplierBatch_inlock();
 
     /**
      * Schedules a fetcher to get the last oplog entry from the sync source.
@@ -576,6 +585,7 @@ private:
     std::unique_ptr<DataReplicatorExternalState> _dataReplicatorExternalState;  // (R)
     executor::TaskExecutor* _exec;                                              // (R)
     StorageInterface* _storage;                                                 // (R)
+    ReplicationProcess* _replicationProcess;                                    // (S)
 
     // This is invoked with the final status of the initial sync. If startup() fails, this callback
     // is never invoked. The caller gets the last applied optime with hash when the initial sync
@@ -605,6 +615,7 @@ private:
     std::unique_ptr<InitialSyncState> _initialSyncState;  // (M)
     std::unique_ptr<OplogFetcher> _oplogFetcher;          // (S)
     std::unique_ptr<Fetcher> _lastOplogEntryFetcher;      // (S)
+    std::unique_ptr<Fetcher> _fCVFetcher;                 // (S)
     std::unique_ptr<MultiApplier> _applier;               // (M)
     HostAndPort _syncSource;                              // (M)
     OpTimeWithHash _lastFetched;                          // (MX)
