@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2014 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -291,6 +293,8 @@ void ReplCoordTest::simulateSuccessfulDryRun(
                                                                << "voteGranted"
                                                                << true)));
             voteRequests++;
+        } else if (consumeHeartbeatV1(noi)) {
+            // The heartbeat has been consumed.
         } else {
             error() << "Black holing unexpected request to " << request.target << ": "
                     << request.cmdObj;
@@ -459,8 +463,8 @@ void ReplCoordTest::signalDrainComplete(OperationContext* opCtx) {
 }
 
 void ReplCoordTest::runSingleNodeElection(OperationContext* opCtx) {
-    getReplCoord()->setMyLastAppliedOpTime(OpTime(Timestamp(1, 0), 0));
-    getReplCoord()->setMyLastDurableOpTime(OpTime(Timestamp(1, 0), 0));
+    getReplCoord()->setMyLastAppliedOpTime(OpTime(Timestamp(1, 1), 0));
+    getReplCoord()->setMyLastDurableOpTime(OpTime(Timestamp(1, 1), 0));
     ASSERT_OK(getReplCoord()->setFollowerMode(MemberState::RS_SECONDARY));
     getReplCoord()->waitForElectionFinish_forTest();
 
@@ -501,21 +505,29 @@ void ReplCoordTest::replyToReceivedHeartbeat() {
 void ReplCoordTest::replyToReceivedHeartbeatV1() {
     NetworkInterfaceMock* net = getNet();
     net->enterNetwork();
-    const NetworkInterfaceMock::NetworkOperationIterator noi = net->getNextReadyRequest();
-    const RemoteCommandRequest& request = noi->getRequest();
-    const ReplSetConfig rsConfig = getReplCoord()->getReplicaSetConfig_forTest();
-    repl::ReplSetHeartbeatArgsV1 hbArgs;
-    ASSERT_OK(hbArgs.initialize(request.cmdObj));
-    repl::ReplSetHeartbeatResponse hbResp;
+    ASSERT(consumeHeartbeatV1(net->getNextReadyRequest()));
+    net->runReadyNetworkOperations();
+    getNet()->exitNetwork();
+}
+
+bool ReplCoordTest::consumeHeartbeatV1(const NetworkInterfaceMock::NetworkOperationIterator& noi) {
+    auto net = getNet();
+    auto& request = noi->getRequest();
+
+    ReplSetHeartbeatArgsV1 args;
+    if (!args.initialize(request.cmdObj).isOK())
+        return false;
+
+    OpTime lastApplied(Timestamp(100, 1), 0);
+    ReplSetHeartbeatResponse hbResp;
+    auto rsConfig = getReplCoord()->getReplicaSetConfig_forTest();
     hbResp.setSetName(rsConfig.getReplSetName());
     hbResp.setState(MemberState::RS_SECONDARY);
     hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setAppliedOpTime(lastApplied);
     BSONObjBuilder respObj;
-    respObj << "ok" << 1;
-    hbResp.addToBSON(&respObj, false);
-    net->scheduleResponse(noi, net->now(), makeResponseStatus(respObj.obj()));
-    net->runReadyNetworkOperations();
-    getNet()->exitNetwork();
+    net->scheduleResponse(noi, net->now(), makeResponseStatus(hbResp.toBSON(true)));
+    return true;
 }
 
 void ReplCoordTest::disableReadConcernMajoritySupport() {
