@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2016 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -50,6 +52,7 @@
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/sharding_state.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/write_concern.h"
 #include "mongo/executor/task_executor.h"
@@ -58,6 +61,15 @@
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
+
+MONGO_EXPORT_SERVER_PARAMETER(rangeDeleterBatchDelayMS, int, 20)
+    ->withValidator([](const int& newVal) {
+        if (newVal < 0) {
+            return Status(ErrorCodes::BadValue, "rangeDeleterBatchDelayMS must not be negative");
+        }
+        return Status::OK();
+    });
+
 namespace {
 
 using Deletion = CollectionRangeDeleter::Deletion;
@@ -65,7 +77,7 @@ using DeleteNotification = CollectionRangeDeleter::DeleteNotification;
 
 const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
                                                 WriteConcernOptions::SyncMode::UNSET,
-                                                Seconds(60));
+                                                WriteConcernOptions::kWriteConcernTimeoutSharding);
 
 boost::optional<DeleteNotification> checkOverlap(std::list<Deletion> const& deletions,
                                                  ChunkRange const& range) {
@@ -203,67 +215,75 @@ boost::optional<Date_t> CollectionRangeDeleter::cleanUpNextRange(
             wrote = e.toStatus();
             warning() << e.what();
         }
-
-        if (!wrote.isOK() || wrote.getValue() == 0) {
-            if (wrote.isOK()) {
-                LOG(0) << "No documents remain to delete in " << nss << " range "
-                       << redact(range->toString());
-            }
-
-            stdx::lock_guard<stdx::mutex> scopedLock(css->_metadataManager->_managerLock);
-            self->_pop(wrote.getStatus());
-            if (!self->_orphans.empty()) {
-                LOG(1) << "Deleting " << nss.ns() << " range "
-                       << redact(self->_orphans.front().range.toString()) << " next.";
-            }
-
-            return Date_t{};
-        }
     }  // drop autoColl
 
+    if (!wrote.isOK() || wrote.getValue() == 0) {
+        if (wrote.isOK()) {
+            LOG(0) << "No documents remain to delete in " << nss << " range "
+                   << redact(range->toString());
+        }
+
+        // Wait for majority replication even when wrote isn't OK or == 0, because it might have
+        // been OK and/or > 0 previously, and the deletions must be persistent before notifying
+        // clients in _pop().
+
+        LOG(0) << "Waiting for majority replication of local deletions in " << nss.ns() << " range "
+               << redact(range->toString());
+
+        repl::ReplClientInfo::forClient(opCtx->getClient()).setLastOpToSystemLastOpTime(opCtx);
+        const auto clientOpTime = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
+
+        // Wait for replication outside the lock
+        const auto status = [&] {
+            try {
+                WriteConcernResult unusedWCResult;
+                return waitForWriteConcern(
+                    opCtx, clientOpTime, kMajorityWriteConcern, &unusedWCResult);
+            } catch (const DBException& e) {
+                return e.toStatus();
+            }
+        }();
+
+        // Get the lock again to finish off this range (including notifying, if necessary).
+        AutoGetCollection autoColl(opCtx, nss, MODE_IX);
+        auto* const css = CollectionShardingState::get(opCtx, nss);
+        auto* const self = forTestOnly ? forTestOnly : &css->_metadataManager->_rangesToClean;
+        stdx::lock_guard<stdx::mutex> scopedLock(css->_metadataManager->_managerLock);
+
+        if (!status.isOK()) {
+            LOG(0) << "Error when waiting for write concern after removing " << nss << " range "
+                   << redact(range->toString()) << " : " << redact(status.reason());
+
+            // If range were already popped (e.g. by dropping nss during the waitForWriteConcern
+            // above) its notification would have been triggered, so this check suffices to ensure
+            // that it is safe to pop the range here
+            if (!notification.ready()) {
+                invariant(!self->isEmpty() && self->_orphans.front().notification == notification);
+                LOG(0) << "Abandoning deletion of latest range in " << nss.ns() << " after local "
+                       << "deletions because of replication failure";
+                self->_pop(status);
+            }
+        } else {
+            LOG(0) << "Finished deleting documents in " << nss.ns() << " range "
+                   << redact(range->toString());
+
+            self->_pop(wrote.getStatus());
+        }
+
+        if (!self->_orphans.empty()) {
+            LOG(1) << "Deleting " << nss.ns() << " range "
+                   << redact(self->_orphans.front().range.toString()) << " next.";
+        }
+
+        return Date_t::now() + stdx::chrono::milliseconds{rangeDeleterBatchDelayMS.load()};
+    }
+
     invariant(range);
     invariantOK(wrote.getStatus());
     invariant(wrote.getValue() > 0);
 
-    repl::ReplClientInfo::forClient(opCtx->getClient()).setLastOpToSystemLastOpTime(opCtx);
-    const auto clientOpTime = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
-
-    // Wait for replication outside the lock
-    const auto status = [&] {
-        try {
-            WriteConcernResult unusedWCResult;
-            return waitForWriteConcern(opCtx, clientOpTime, kMajorityWriteConcern, &unusedWCResult);
-        } catch (const DBException& e) {
-            return e.toStatus();
-        }
-    }();
-
-    if (!status.isOK()) {
-        LOG(0) << "Error when waiting for write concern after removing " << nss << " range "
-               << redact(range->toString()) << " : " << redact(status.reason());
-
-        AutoGetCollection autoColl(opCtx, nss, MODE_IX);
-        auto* const css = CollectionShardingState::get(opCtx, nss);
-
-        stdx::lock_guard<stdx::mutex> scopedLock(css->_metadataManager->_managerLock);
-        auto* const self = &css->_metadataManager->_rangesToClean;
-
-        // If range were already popped (e.g. by dropping nss during the waitForWriteConcern above)
-        // its notification would have been triggered, so this check suffices to ensure that it is
-        // safe to pop the range here
-        if (!notification.ready()) {
-            invariant(!self->isEmpty() && self->_orphans.front().notification == notification);
-            LOG(0) << "Abandoning deletion of latest range in " << nss.ns() << " after "
-                   << wrote.getValue() << " local deletions because of replication failure";
-            self->_pop(status);
-        }
-    } else {
-        LOG(1) << "Deleted " << wrote.getValue() << " documents in " << nss.ns() << " range "
-               << redact(range->toString());
-    }
-
     notification.abandon();
-    return Date_t{};
+    return Date_t::now() + stdx::chrono::milliseconds{rangeDeleterBatchDelayMS.load()};
 }
 
 StatusWith<int> CollectionRangeDeleter::_doDeletion(OperationContext* opCtx,
@@ -312,16 +332,16 @@ StatusWith<int> CollectionRangeDeleter::_doDeletion(OperationContext* opCtx,
         saver.emplace("moveChunk", nss.ns(), "cleaning");
     }
 
+    auto halfOpen = BoundInclusion::kIncludeStartKeyOnly;
+    auto manual = PlanExecutor::YIELD_MANUAL;
+    auto forward = InternalPlanner::FORWARD;
+    auto fetch = InternalPlanner::IXSCAN_FETCH;
+
+    auto exec = InternalPlanner::indexScan(
+        opCtx, collection, descriptor, min, max, halfOpen, manual, forward, fetch);
+
     int numDeleted = 0;
     do {
-        auto halfOpen = BoundInclusion::kIncludeStartKeyOnly;
-        auto manual = PlanExecutor::YIELD_MANUAL;
-        auto forward = InternalPlanner::FORWARD;
-        auto fetch = InternalPlanner::IXSCAN_FETCH;
-
-        auto exec = InternalPlanner::indexScan(
-            opCtx, collection, descriptor, min, max, halfOpen, manual, forward, fetch);
-
         RecordId rloc;
         BSONObj obj;
         PlanExecutor::ExecState state = exec->getNext(&obj, &rloc);
@@ -337,6 +357,7 @@ StatusWith<int> CollectionRangeDeleter::_doDeletion(OperationContext* opCtx,
         }
         invariant(PlanExecutor::ADVANCED == state);
 
+        exec->saveState();
         writeConflictRetry(opCtx, "delete range", nss.ns(), [&] {
             WriteUnitOfWork wuow(opCtx);
             if (saver) {
@@ -345,6 +366,15 @@ StatusWith<int> CollectionRangeDeleter::_doDeletion(OperationContext* opCtx,
             collection->deleteDocument(opCtx, kUninitializedStmtId, rloc, nullptr, true);
             wuow.commit();
         });
+        auto restoreStateStatus = exec->restoreState();
+        if (!restoreStateStatus.isOK()) {
+            warning() << "error restoring cursor state while trying to delete " << redact(min)
+                      << " to " << redact(max) << " in " << nss
+                      << ", stats: " << Explain::getWinningPlanStats(exec.get()) << ": "
+                      << redact(restoreStateStatus);
+            break;
+        }
+
     } while (++numDeleted < maxToDelete);
 
     return numDeleted;
