@@ -1,29 +1,31 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
@@ -38,6 +40,8 @@
 #include "mongo/db/catalog/catalog_raii.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
+#include "mongo/db/repl/replication_process.h"
+#include "mongo/db/s/active_migrations_registry.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/migration_chunk_cloner_source_legacy.h"
 #include "mongo/db/s/migration_source_manager.h"
@@ -75,7 +79,7 @@ public:
         auto css = CollectionShardingState::get(opCtx, *nss);
         uassert(ErrorCodes::IllegalOperation,
                 str::stream() << "No active migrations were found for collection " << nss->ns(),
-                css && css->getMigrationSourceManager());
+                css->getMigrationSourceManager());
 
         // It is now safe to access the cloner
         _chunkCloner = dynamic_cast<MigrationChunkClonerSourceLegacy*>(
@@ -261,7 +265,7 @@ public:
 
         BSONArrayBuilder arrBuilder;
 
-        repl::OpTime opTime;
+        boost::optional<repl::OpTime> opTime;
 
         writeConflictRetry(opCtx,
                            "Fetching session related oplogs for migration",
@@ -272,10 +276,31 @@ public:
                                    opCtx, &arrBuilder);
                            });
 
-        WriteConcernResult wcResult;
-        WriteConcernOptions majorityWC(
-            WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, 0);
-        uassertStatusOK(waitForWriteConcern(opCtx, opTime, majorityWC, &wcResult));
+        // If the batch returns something, we wait for write concern to ensure that all the entries
+        // in the batch have been majority committed. We then need to check that the rollback id
+        // hasn't changed since we started migration, because a change would indicate that some data
+        // in this batch may have been rolled back. In this case, we abort the migration.
+        if (opTime) {
+            WriteConcernResult wcResult;
+            WriteConcernOptions majorityWC(
+                WriteConcernOptions::kMajority, WriteConcernOptions::SyncMode::UNSET, 0);
+            uassertStatusOK(waitForWriteConcern(opCtx, opTime.get(), majorityWC, &wcResult));
+
+            auto rollbackIdAtMigrationInit = [&]() {
+                AutoGetActiveCloner autoCloner(opCtx, migrationSessionId);
+                return autoCloner.getCloner()->getRollbackIdAtInit();
+            }();
+
+            // The check for rollback id must be done after having waited for majority in order to
+            // ensure that whatever was waited on didn't get rolled back.
+            auto rollbackId = repl::ReplicationProcess::get(opCtx)->getRollbackID();
+            uassert(50881,
+                    str::stream() << "rollback detected, rollbackId was "
+                                  << rollbackIdAtMigrationInit
+                                  << " but is now "
+                                  << rollbackId,
+                    rollbackId == rollbackIdAtMigrationInit);
+        }
 
         result.appendArray("oplog", arrBuilder.arr());
         return true;
