@@ -1,29 +1,31 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
@@ -46,6 +48,7 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/delete.h"
+#include "mongo/db/ops/write_ops_exec.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator_global.h"
 #include "mongo/db/s/collection_sharding_state.h"
@@ -62,12 +65,10 @@
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
+#include "mongo/util/producer_consumer_queue.h"
+#include "mongo/util/scopeguard.h"
 
 namespace mongo {
-
-using std::string;
-using str::stream;
-
 namespace {
 
 const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
@@ -82,7 +83,7 @@ const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
 /**
  * Returns a human-readabale name of the migration manager's state.
  */
-string stateToString(MigrationDestinationManager::State state) {
+std::string stateToString(MigrationDestinationManager::State state) {
     switch (state) {
         case MigrationDestinationManager::READY:
             return "ready";
@@ -222,25 +223,28 @@ MigrationDestinationManager::State MigrationDestinationManager::getState() const
 void MigrationDestinationManager::setState(State newState) {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
     _state = newState;
+    _stateChangedCV.notify_all();
 }
 
-void MigrationDestinationManager::setStateFail(std::string msg) {
+void MigrationDestinationManager::_setStateFail(StringData msg) {
     log() << msg;
     {
         stdx::lock_guard<stdx::mutex> sl(_mutex);
-        _errmsg = std::move(msg);
+        _errmsg = msg.toString();
         _state = FAIL;
+        _stateChangedCV.notify_all();
     }
 
     _sessionMigration->forceFail(msg);
 }
 
-void MigrationDestinationManager::setStateFailWarn(std::string msg) {
+void MigrationDestinationManager::_setStateFailWarn(StringData msg) {
     warning() << msg;
     {
         stdx::lock_guard<stdx::mutex> sl(_mutex);
-        _errmsg = std::move(msg);
+        _errmsg = msg.toString();
         _state = FAIL;
+        _stateChangedCV.notify_all();
     }
 
     _sessionMigration->forceFail(msg);
@@ -255,7 +259,21 @@ bool MigrationDestinationManager::_isActive(WithLock) const {
     return _sessionId.is_initialized();
 }
 
-void MigrationDestinationManager::report(BSONObjBuilder& b) {
+void MigrationDestinationManager::report(BSONObjBuilder& b,
+                                         OperationContext* opCtx,
+                                         bool waitForSteadyOrDone) {
+    if (waitForSteadyOrDone) {
+        stdx::unique_lock<stdx::mutex> lock(_mutex);
+        try {
+            opCtx->waitForConditionOrInterruptFor(_stateChangedCV, lock, Seconds(1), [&]() -> bool {
+                return _state != READY && _state != CLONE && _state != CATCHUP;
+            });
+        } catch (...) {
+            // Ignoring this error because this is an optional parameter and we catch timeout
+            // exceptions later.
+        }
+        b.append("waited", true);
+    }
     stdx::lock_guard<stdx::mutex> sl(_mutex);
 
     b.appendBool("active", _sessionId.is_initialized());
@@ -311,6 +329,7 @@ Status MigrationDestinationManager::start(const NamespaceString& nss,
     invariant(!_scopedRegisterReceiveChunk);
 
     _state = READY;
+    _stateChangedCV.notify_all();
     _errmsg = "";
 
     _nss = nss;
@@ -349,8 +368,54 @@ Status MigrationDestinationManager::start(const NamespaceString& nss,
     return Status::OK();
 }
 
-Status MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
+void MigrationDestinationManager::cloneDocumentsFromDonor(
+    OperationContext* opCtx,
+    stdx::function<void(OperationContext*, BSONObj)> insertBatchFn,
+    stdx::function<BSONObj(OperationContext*)> fetchBatchFn) {
 
+    ProducerConsumerQueue<BSONObj> batches(1);
+    stdx::thread inserterThread{[&] {
+        Client::initThreadIfNotAlready("chunkInserter");
+        auto inserterOpCtx = Client::getCurrent()->makeOperationContext();
+        auto consumerGuard = MakeGuard([&] { batches.closeConsumerEnd(); });
+        try {
+            while (true) {
+                auto nextBatch = batches.pop(inserterOpCtx.get());
+                auto arr = nextBatch["objects"].Obj();
+                if (arr.isEmpty()) {
+                    return;
+                }
+                insertBatchFn(inserterOpCtx.get(), arr);
+            }
+        } catch (...) {
+            stdx::lock_guard<Client> lk(*opCtx->getClient());
+            opCtx->getServiceContext()->killOperation(opCtx, exceptionToStatus().code());
+            log() << "Batch insertion failed " << causedBy(redact(exceptionToStatus()));
+        }
+    }};
+    auto inserterThreadJoinGuard = MakeGuard([&] {
+        batches.closeProducerEnd();
+        inserterThread.join();
+    });
+
+    while (true) {
+        opCtx->checkForInterrupt();
+
+        auto res = fetchBatchFn(opCtx);
+
+        opCtx->checkForInterrupt();
+        batches.push(res.getOwned(), opCtx);
+        auto arr = res["objects"].Obj();
+        if (arr.isEmpty()) {
+            inserterThreadJoinGuard.Dismiss();
+            inserterThread.join();
+            opCtx->checkForInterrupt();
+            break;
+        }
+    }
+}
+
+Status MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
 
     if (!_sessionId) {
@@ -366,6 +431,7 @@ Status MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
     }
 
     _state = ABORT;
+    _stateChangedCV.notify_all();
     _errmsg = "aborted";
 
     return Status::OK();
@@ -374,6 +440,7 @@ Status MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
 void MigrationDestinationManager::abortWithoutSessionIdCheck() {
     stdx::lock_guard<stdx::mutex> sl(_mutex);
     _state = ABORT;
+    _stateChangedCV.notify_all();
     _errmsg = "aborted without session id check";
 }
 
@@ -406,6 +473,7 @@ Status MigrationDestinationManager::startCommit(const MigrationSessionId& sessio
 
     _sessionMigration->finish();
     _state = COMMIT_START;
+    _stateChangedCV.notify_all();
 
     auto const deadline = Date_t::now() + Seconds(30);
     while (_sessionId) {
@@ -413,6 +481,7 @@ Status MigrationDestinationManager::startCommit(const MigrationSessionId& sessio
             _isActiveCV.wait_until(lock, deadline.toSystemTimePoint())) {
             _errmsg = str::stream() << "startCommit timed out waiting, " << _sessionId->toString();
             _state = FAIL;
+            _stateChangedCV.notify_all();
             return {ErrorCodes::CommandFailed, _errmsg};
         }
     }
@@ -440,10 +509,8 @@ void MigrationDestinationManager::_migrateThread(BSONObj min,
     try {
         _migrateDriver(
             opCtx.get(), min, max, shardKeyPattern, fromShardConnString, epoch, writeConcern);
-    } catch (std::exception& e) {
-        setStateFail(str::stream() << "migrate failed: " << redact(e.what()));
     } catch (...) {
-        setStateFail("migrate failed with unknown exception: UNKNOWN ERROR");
+        _setStateFail(str::stream() << "migrate failed: " << redact(exceptionToStatus()));
     }
 
     if (getState() != DONE && !MONGO_FAIL_POINT(failMigrationLeaveOrphans)) {
@@ -520,12 +587,12 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
             conn->getCollectionInfos(_nss.db().toString(), BSON("name" << _nss.coll()));
 
         if (infos.size() != 1) {
-            setStateFailWarn(str::stream()
-                             << "expected listCollections against the donor shard for "
-                             << _nss.ns()
-                             << " to return 1 entry, but got "
-                             << infos.size()
-                             << " entries");
+            _setStateFailWarn(str::stream()
+                              << "expected listCollections against the donor shard for "
+                              << _nss.ns()
+                              << " to return 1 entry, but got "
+                              << infos.size()
+                              << " entries");
             return;
         }
 
@@ -545,15 +612,15 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
                 info = entry["info"].Obj();
             }
             if (info["uuid"].eoo()) {
-                setStateFailWarn(str::stream()
-                                 << "The donor shard did not return a UUID for collection "
-                                 << _nss.ns()
-                                 << " as part of its listCollections response: "
-                                 << entry
-                                 << ", but this node expects to see a UUID since its "
-                                    "feature compatibility version is 3.6. Please follow "
-                                    "the online documentation to set the same feature "
-                                    "compatibility version across the cluster.");
+                _setStateFailWarn(str::stream()
+                                  << "The donor shard did not return a UUID for collection "
+                                  << _nss.ns()
+                                  << " as part of its listCollections response: "
+                                  << entry
+                                  << ", but this node expects to see a UUID since its "
+                                     "feature compatibility version is 3.6. Please follow "
+                                     "the online documentation to set the same feature "
+                                     "compatibility version across the cluster.");
                 return;
             }
             donorOptionsBob.append(info["uuid"]);
@@ -576,8 +643,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
 
         OldClientWriteContext ctx(opCtx, _nss.ns());
         if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, _nss)) {
-            setStateFailWarn(str::stream() << "Not primary during migration: " << _nss.ns()
-                                           << ": checking if collection exists");
+            _setStateFailWarn(str::stream() << "Not primary during migration: " << _nss.ns()
+                                            << ": checking if collection exists");
             return;
         }
 
@@ -592,10 +659,10 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
             }
 
             if (!collection->getCatalogEntry()->isEqualToMetadataUUID(opCtx, donorUUID)) {
-                setStateFailWarn(
+                _setStateFailWarn(
                     str::stream()
                     << "Cannot receive chunk "
-                    << ChunkRange(min, max).toString()
+                    << redact(ChunkRange(min, max).toString())
                     << " for collection "
                     << _nss.ns()
                     << " because we already have an identically named collection with UUID "
@@ -633,19 +700,19 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
         if (!donorIndexSpecs.empty()) {
             // Only copy indexes if the collection does not have any documents.
             if (collection->numRecords(opCtx) > 0) {
-                setStateFailWarn(str::stream() << "aborting migration, shard is missing "
-                                               << donorIndexSpecs.size()
-                                               << " indexes and "
-                                               << "collection is not empty. Non-trivial "
-                                               << "index creation should be scheduled manually");
+                _setStateFailWarn(str::stream() << "aborting migration, shard is missing "
+                                                << donorIndexSpecs.size()
+                                                << " indexes and "
+                                                << "collection is not empty. Non-trivial "
+                                                << "index creation should be scheduled manually");
                 return;
             }
 
             auto indexInfoObjs = indexer.init(donorIndexSpecs);
             if (!indexInfoObjs.isOK()) {
-                setStateFailWarn(str::stream() << "failed to create index before migrating data. "
-                                               << " error: "
-                                               << redact(indexInfoObjs.getStatus()));
+                _setStateFailWarn(str::stream() << "failed to create index before migrating data. "
+                                                << " error: "
+                                                << redact(indexInfoObjs.getStatus()));
                 return;
             }
 
@@ -673,14 +740,14 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
         auto notification = _notePending(opCtx, _nss, epoch, footprint);
         // Wait for the range deletion to report back
         if (!notification.waitStatus(opCtx).isOK()) {
-            setStateFail(notification.waitStatus(opCtx).reason());
+            _setStateFail(redact(notification.waitStatus(opCtx).reason()));
             return;
         }
 
         // Wait for any other, overlapping queued deletions to drain
         auto status = CollectionShardingState::waitForClean(opCtx, _nss, epoch, footprint);
         if (!status.isOK()) {
-            setStateFail(status.reason());
+            _setStateFail(redact(status.reason()));
             return;
         }
 
@@ -698,87 +765,77 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
 
         _chunkMarkedPending = true;  // no lock needed, only the migrate thread looks.
 
-        while (true) {
+        auto assertNotAborted = [&](OperationContext* opCtx) {
+            opCtx->checkForInterrupt();
+            uassert(50664, "Migration aborted while copying documents", getState() != ABORT);
+        };
+
+        auto insertBatchFn = [&](OperationContext* opCtx, BSONObj arr) {
+            int batchNumCloned = 0;
+            int batchClonedBytes = 0;
+
+            assertNotAborted(opCtx);
+
+            write_ops::Insert insertOp(_nss);
+            insertOp.getWriteCommandBase().setOrdered(true);
+            insertOp.setDocuments([&] {
+                std::vector<BSONObj> toInsert;
+                for (const auto& doc : arr) {
+                    BSONObj docToClone = doc.Obj();
+                    toInsert.push_back(docToClone);
+                    batchNumCloned++;
+                    batchClonedBytes += docToClone.objsize();
+                }
+                return toInsert;
+            }());
+
+            const WriteResult reply = performInserts(opCtx, insertOp, true);
+
+            for (unsigned long i = 0; i < reply.results.size(); ++i) {
+                uassertStatusOK(reply.results[i]);
+            }
+
+            {
+                stdx::lock_guard<stdx::mutex> statsLock(_mutex);
+                _numCloned += batchNumCloned;
+                _clonedBytes += batchClonedBytes;
+            }
+            if (writeConcern.shouldWaitForOtherNodes()) {
+                repl::ReplicationCoordinator::StatusAndDuration replStatus =
+                    repl::ReplicationCoordinator::get(opCtx)->awaitReplication(
+                        opCtx,
+                        repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
+                        writeConcern);
+                if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
+                    warning() << "secondaryThrottle on, but doc insert timed out; "
+                                 "continuing";
+                } else {
+                    uassertStatusOK(replStatus.status);
+                }
+            }
+        };
+
+        auto fetchBatchFn = [&](OperationContext* opCtx) {
             BSONObj res;
             if (!conn->runCommand("admin",
                                   migrateCloneRequest,
                                   res)) {  // gets array of objects to copy, in disk order
-                setStateFail(str::stream() << "_migrateClone failed: " << redact(res.toString()));
                 conn.done();
-                return;
+                const std::string errMsg = str::stream() << "_migrateClone failed: "
+                                                         << redact(res.toString());
+                uasserted(50665, errMsg);
             }
+            return res;
+        };
 
-            BSONObj arr = res["objects"].Obj();
-            int thisTime = 0;
-
-            BSONObjIterator i(arr);
-            while (i.more()) {
-                opCtx->checkForInterrupt();
-
-                if (getState() == ABORT) {
-                    log() << "Migration aborted while copying documents";
-                    return;
-                }
-
-                BSONObj docToClone = i.next().Obj();
-                {
-                    OldClientWriteContext cx(opCtx, _nss.ns());
-
-                    BSONObj localDoc;
-                    if (willOverrideLocalId(opCtx,
-                                            _nss,
-                                            min,
-                                            max,
-                                            shardKeyPattern,
-                                            cx.db(),
-                                            docToClone,
-                                            &localDoc)) {
-                        string errMsg = str::stream() << "cannot migrate chunk, local document "
-                                                      << redact(localDoc)
-                                                      << " has same _id as cloned "
-                                                      << "remote document " << redact(docToClone);
-
-                        warning() << errMsg;
-
-                        // Exception will abort migration cleanly
-                        uasserted(16976, errMsg);
-                    }
-
-                    Helpers::upsert(opCtx, _nss.ns(), docToClone, true);
-                }
-                thisTime++;
-
-                {
-                    stdx::lock_guard<stdx::mutex> statsLock(_mutex);
-                    _numCloned++;
-                    _clonedBytes += docToClone.objsize();
-                }
-
-                if (writeConcern.shouldWaitForOtherNodes()) {
-                    repl::ReplicationCoordinator::StatusAndDuration replStatus =
-                        repl::getGlobalReplicationCoordinator()->awaitReplication(
-                            opCtx,
-                            repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp(),
-                            writeConcern);
-                    if (replStatus.status.code() == ErrorCodes::WriteConcernFailed) {
-                        warning() << "secondaryThrottle on, but doc insert timed out; "
-                                     "continuing";
-                    } else {
-                        massertStatusOK(replStatus.status);
-                    }
-                }
-            }
-
-            if (thisTime == 0)
-                break;
-        }
+        cloneDocumentsFromDonor(opCtx, insertBatchFn, fetchBatchFn);
 
         timing.done(3);
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep3);
 
         if (MONGO_FAIL_POINT(failMigrationLeaveOrphans)) {
-            setStateFail(str::stream() << "failing migration after cloning " << _numCloned
-                                       << " docs due to failMigrationLeaveOrphans failpoint");
+            _setStateFail(str::stream() << "failing migration after cloning " << _numCloned
+                                        << " docs due to failMigrationLeaveOrphans failpoint");
             return;
         }
     }
@@ -796,7 +853,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
         while (true) {
             BSONObj res;
             if (!conn->runCommand("admin", xferModsRequest, res)) {
-                setStateFail(str::stream() << "_transferMods failed: " << redact(res));
+                _setStateFail(str::stream() << "_transferMods failed: " << redact(res));
                 conn.done();
                 return;
             }
@@ -829,7 +886,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
             }
 
             if (i == maxIterations) {
-                setStateFail("secondary can't keep up with migrate");
+                _setStateFail("secondary can't keep up with migrate");
                 conn.done();
                 return;
             }
@@ -861,7 +918,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
         }
 
         if (t.minutes() >= 600) {
-            setStateFail("Cannot go to critical section because secondaries cannot keep up");
+            _setStateFail("Cannot go to critical section because secondaries cannot keep up");
             return;
         }
     }
@@ -884,8 +941,8 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
 
             BSONObj res;
             if (!conn->runCommand("admin", xferModsRequest, res)) {
-                setStateFail(str::stream() << "_transferMods failed in STEADY state: "
-                                           << redact(res));
+                _setStateFail(str::stream() << "_transferMods failed in STEADY state: "
+                                            << redact(res));
                 conn.done();
                 return;
             }
@@ -915,7 +972,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
         }
 
         if (getState() == FAIL) {
-            setStateFail("timed out waiting for commit");
+            _setStateFail("timed out waiting for commit");
             return;
         }
 
@@ -925,7 +982,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx,
 
     _sessionMigration->join();
     if (_sessionMigration->getState() == SessionCatalogMigrationDestination::State::ErrorOccurred) {
-        setStateFail(_sessionMigration->getErrMsg());
+        _setStateFail(redact(_sessionMigration->getErrMsg()));
         return;
     }
 
@@ -1008,10 +1065,9 @@ bool MigrationDestinationManager::_applyMigrateOp(OperationContext* opCtx,
             BSONObj localDoc;
             if (willOverrideLocalId(
                     opCtx, nss, min, max, shardKeyPattern, cx.db(), updatedDoc, &localDoc)) {
-                string errMsg = str::stream() << "cannot migrate chunk, local document " << localDoc
-                                              << " has same _id as reloaded remote document "
-                                              << updatedDoc;
-
+                const std::string errMsg = str::stream()
+                    << "cannot migrate chunk, local document " << redact(localDoc)
+                    << " has same _id as reloaded remote document " << redact(updatedDoc);
                 warning() << errMsg;
 
                 // Exception will abort migration cleanly
@@ -1042,8 +1098,8 @@ bool MigrationDestinationManager::_flushPendingWrites(OperationContext* opCtx,
         return false;
     }
 
-    log() << "migrate commit succeeded flushing to secondaries for '" << ns << "' " << min << " -> "
-          << max;
+    log() << "migrate commit succeeded flushing to secondaries for '" << ns << "' " << redact(min)
+          << " -> " << redact(max);
 
     return true;
 }
@@ -1062,7 +1118,7 @@ CollectionShardingState::CleanupNotification MigrationDestinationManager::_noteP
     // checking this here is that in the future we shouldn't have this problem.
     if (!metadata || metadata->getCollVersion().epoch() != epoch) {
         return Status{ErrorCodes::StaleShardVersion,
-                      str::stream() << "not noting chunk " << range.toString()
+                      str::stream() << "not noting chunk " << redact(range.toString())
                                     << " as pending because the epoch of "
                                     << nss.ns()
                                     << " changed"};
@@ -1072,9 +1128,10 @@ CollectionShardingState::CleanupNotification MigrationDestinationManager::_noteP
     auto notification = css->beginReceive(range);
     if (notification.ready() && !notification.waitStatus(opCtx).isOK()) {
         return Status{notification.waitStatus(opCtx).code(),
-                      str::stream() << "Collection " << nss.ns() << " range " << range.toString()
+                      str::stream() << "Collection " << nss.ns() << " range "
+                                    << redact(range.toString())
                                     << " migration aborted: "
-                                    << notification.waitStatus(opCtx).reason()};
+                                    << redact(notification.waitStatus(opCtx).reason())};
     }
     return notification;
 }
