@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2017 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -88,6 +90,9 @@ Status persistCollectionAndChangedChunks(OperationContext* opCtx,
                                                      collAndChunks.shardKeyPattern,
                                                      collAndChunks.defaultCollation,
                                                      collAndChunks.shardKeyIsUnique);
+    // Mark the chunk metadata as refreshing, so that secondaries are aware of refresh.
+    update.setRefreshing(true);
+
     Status status = updateShardCollectionsEntry(opCtx,
                                                 BSON(ShardCollectionType::ns() << nss.ns()),
                                                 update.toBSON(),
@@ -97,12 +102,6 @@ Status persistCollectionAndChangedChunks(OperationContext* opCtx,
         return status;
     }
 
-    // Mark the chunk metadata as refreshing, so that secondaries are aware of refresh.
-    status = setPersistedRefreshFlags(opCtx, nss);
-    if (!status.isOK()) {
-        return status;
-    }
-
     // Update the chunks.
     status = updateShardChunks(opCtx, nss, collAndChunks.changedChunks, collAndChunks.epoch);
     if (!status.isOK()) {
@@ -488,8 +487,9 @@ void ShardServerCatalogCacheLoader::_schedulePrimaryGetChunksSince(
                 return;
             }
 
-            log() << "Cache loader remotely refreshed for collection " << nss << " from version "
-                  << maxLoaderVersion << " and no metadata was found.";
+            LOG_CATALOG_REFRESH(1) << "Cache loader remotely refreshed for collection " << nss
+                                   << " from version " << maxLoaderVersion
+                                   << " and no metadata was found.";
         } else if (swCollectionAndChangedChunks.isOK()) {
             auto& collAndChunks = swCollectionAndChangedChunks.getValue();
 
@@ -518,10 +518,10 @@ void ShardServerCatalogCacheLoader::_schedulePrimaryGetChunksSince(
                     }
                 }
 
-                log() << "Cache loader remotely refreshed for collection " << nss
-                      << " from collection version " << maxLoaderVersion
-                      << " and found collection version "
-                      << collAndChunks.changedChunks.back().getVersion();
+                LOG_CATALOG_REFRESH(1) << "Cache loader remotely refreshed for collection " << nss
+                                       << " from collection version " << maxLoaderVersion
+                                       << " and found collection version "
+                                       << collAndChunks.changedChunks.back().getVersion();
 
                 // Metadata was found remotely -- otherwise would have received NamespaceNotFound
                 // rather than Status::OK(). Return metadata for CatalogCache that's GTE
@@ -571,18 +571,19 @@ StatusWith<CollectionAndChangedChunks> ShardServerCatalogCacheLoader::_getLoader
         persisted = std::move(swPersisted.getValue());
     }
 
-    log() << "Cache loader found "
-          << (enqueued.changedChunks.empty()
-                  ? (tasksAreEnqueued ? "a drop enqueued" : "no enqueued metadata")
-                  : ("enqueued metadata from " +
-                     enqueued.changedChunks.front().getVersion().toString() + " to " +
-                     enqueued.changedChunks.back().getVersion().toString()))
-          << " and " << (persisted.changedChunks.empty()
-                             ? "no persisted metadata"
-                             : ("persisted metadata from " +
-                                persisted.changedChunks.front().getVersion().toString() + " to " +
-                                persisted.changedChunks.back().getVersion().toString()))
-          << ", GTE cache version " << catalogCacheSinceVersion;
+    LOG_CATALOG_REFRESH(1)
+        << "Cache loader found "
+        << (enqueued.changedChunks.empty()
+                ? (tasksAreEnqueued ? "a drop enqueued" : "no enqueued metadata")
+                : ("enqueued metadata from " +
+                   enqueued.changedChunks.front().getVersion().toString() + " to " +
+                   enqueued.changedChunks.back().getVersion().toString()))
+        << " and " << (persisted.changedChunks.empty()
+                           ? "no persisted metadata"
+                           : ("persisted metadata from " +
+                              persisted.changedChunks.front().getVersion().toString() + " to " +
+                              persisted.changedChunks.back().getVersion().toString()))
+        << ", GTE cache version " << catalogCacheSinceVersion;
 
     if (!tasksAreEnqueued) {
         // There are no tasks in the queue. Return the persisted metadata.
@@ -658,33 +659,25 @@ std::pair<bool, CollectionAndChangedChunks> ShardServerCatalogCacheLoader::_getE
 
 Status ShardServerCatalogCacheLoader::_ensureMajorityPrimaryAndScheduleTask(
     OperationContext* opCtx, const NamespaceString& nss, Task task) {
-    Status linearizableReadStatus = waitForLinearizableReadConcern(opCtx);
-    if (!linearizableReadStatus.isOK()) {
-        return {linearizableReadStatus.code(),
-                str::stream() << "Unable to schedule routing table update because this is not the"
-                              << " majority primary and may not have the latest data. Error: "
-                              << linearizableReadStatus.reason()};
-    }
 
     stdx::lock_guard<stdx::mutex> lock(_mutex);
-
     const bool wasEmpty = _taskLists[nss].empty();
     _taskLists[nss].addTask(std::move(task));
-
-    if (wasEmpty) {
-        Status status = _threadPool.schedule([this, nss]() { _runTasks(nss); });
-        if (!status.isOK()) {
-            log() << "Cache loader failed to schedule persisted metadata update"
-                  << " task for namespace '" << nss << "' due to '" << redact(status)
-                  << "'. Clearing task list so that scheduling"
-                  << " will be attempted by the next caller to refresh this namespace.";
-            stdx::lock_guard<stdx::mutex> lock(_mutex);
-            _taskLists.erase(nss);
-        }
-        return status;
+    if (!wasEmpty) {
+        return Status::OK();
     }
 
-    return Status::OK();
+    Status status = _threadPool.schedule([this, nss]() { _runTasks(nss); });
+    if (!status.isOK()) {
+        log() << "Cache loader failed to schedule persisted metadata update"
+              << " task for namespace '" << nss << "' due to '" << redact(status)
+              << "'. Clearing task list so that scheduling"
+              << " will be attempted by the next caller to refresh this namespace.";
+
+        _taskLists.erase(nss);
+    }
+
+    return status;
 }
 
 void ShardServerCatalogCacheLoader::_runTasks(const NamespaceString& nss) {
@@ -773,8 +766,9 @@ void ShardServerCatalogCacheLoader::_updatePersistedMetadata(OperationContext* o
                           << "'. Will be retried.",
             status.isOK());
 
-    LOG(1) << "Successfully updated persisted chunk metadata for collection '" << nss << "' from '"
-           << task.minQueryVersion << "' to collection version '" << task.maxQueryVersion << "'.";
+    LOG_CATALOG_REFRESH(1) << "Successfully updated persisted chunk metadata for collection '"
+                           << nss << "' from '" << task.minQueryVersion
+                           << "' to collection version '" << task.maxQueryVersion << "'.";
 }
 
 CollectionAndChangedChunks
@@ -809,9 +803,10 @@ ShardServerCatalogCacheLoader::_getCompletePersistedMetadataForSecondarySinceVer
             return collAndChangedChunks;
         }
 
-        LOG(1) << "Cache loader read meatadata while updates were being applied: this metadata may"
-               << " be incomplete. Retrying. Refresh state before read: " << beginRefreshState
-               << ". Current refresh state: '" << endRefreshState << "'.";
+        LOG_CATALOG_REFRESH(1)
+            << "Cache loader read meatadata while updates were being applied: this metadata may"
+            << " be incomplete. Retrying. Refresh state before read: " << beginRefreshState
+            << ". Current refresh state: '" << endRefreshState << "'.";
     }
 }
 
