@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -29,14 +31,19 @@
 #pragma once
 
 #include <memory>
+#include <utility>
 #include <vector>
 
 #include "mongo/db/cursor_id.h"
+#include "mongo/db/generic_cursor.h"
+#include "mongo/db/kill_sessions.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/session_killer.h"
 #include "mongo/platform/random.h"
 #include "mongo/s/query/cluster_client_cursor.h"
 #include "mongo/stdx/mutex.h"
 #include "mongo/stdx/unordered_map.h"
+#include "mongo/util/concurrency/with_lock.h"
 #include "mongo/util/time_support.h"
 
 namespace mongo {
@@ -76,11 +83,11 @@ public:
     //
 
     enum class CursorType {
-        // Represents a cursor operating on an unsharded namespace.
-        NamespaceNotSharded,
+        // Represents a cursor retrieving data from a single remote source.
+        SingleTarget,
 
-        // Represents a cursor operating on a sharded namespace.
-        NamespaceSharded,
+        // Represents a cursor retrieving data from multiple remote sources.
+        MultiTarget,
     };
 
     enum class CursorLifetime {
@@ -100,11 +107,11 @@ public:
     };
 
     struct Stats {
-        // Count of open cursors registered with CursorType::NamespaceSharded.
-        size_t cursorsSharded = 0;
+        // Count of open cursors registered with CursorType::MultiTarget.
+        size_t cursorsMultiTarget = 0;
 
-        // Count of open cursors registered with CursorType::NamespaceNotSharded.
-        size_t cursorsNotSharded = 0;
+        // Count of open cursors registered with CursorType::SingleTarget.
+        size_t cursorsSingleTarget = 0;
 
         // Count of pinned cursors.
         size_t cursorsPinned = 0;
@@ -154,7 +161,18 @@ public:
          *
          * Can block.
          */
-        StatusWith<ClusterQueryResult> next();
+        StatusWith<ClusterQueryResult> next(RouterExecStage::ExecContext);
+
+        /**
+         * Sets the operation context for the cursor. Must be called before the first call to
+         * next().
+         */
+        void reattachToOperationContext(OperationContext* opCtx);
+
+        /**
+         * Detaches the cursor from its current OperationContext.
+         */
+        void detachFromOperationContext();
 
         /**
          * Returns whether or not the underlying cursor is tailing a capped collection.  Cannot be
@@ -162,6 +180,19 @@ public:
          */
         bool isTailable() const;
 
+        /**
+         * Returns whether or not the underlying cursor is tailing a capped collection and was
+         * created with the 'awaitData' flag set.  Cannot be called after returnCursor() is called.
+         * A cursor must be owned.
+         */
+        bool isTailableAndAwaitData() const;
+
+        /**
+         * Returns the set of authenticated users when this cursor was created. Cannot be called
+         * after returnCursor() is called.  A cursor must be owned.
+         */
+        UserNameIterator getAuthenticatedUsers() const;
+
         /**
          * Transfers ownership of the underlying cursor back to the manager.  A cursor must be
          * owned, and a cursor will no longer be owned after this method completes.
@@ -176,6 +207,11 @@ public:
          */
         CursorId getCursorId() const;
 
+        /**
+         * Returns the read preference setting for this cursor.
+         */
+        boost::optional<ReadPreferenceSetting> getReadPreference() const;
+
         /**
          * Returns the number of result documents returned so far by this cursor via the next()
          * method.
@@ -202,14 +238,6 @@ public:
          */
         Status setAwaitDataTimeout(Milliseconds awaitDataTimeout);
 
-
-        /**
-         * Update the operation context for remote requests.
-         *
-         * Network requests depend on having a valid operation context for user initiated actions.
-         */
-        void setOperationContext(OperationContext* txn);
-
     private:
         // ClusterCursorManager is a friend so that its methods can call the PinnedCursor
         // constructor declared below, which is private to prevent clients from calling it directly.
@@ -254,7 +282,7 @@ public:
      * Kills and reaps all cursors currently owned by this cursor manager, and puts the manager
      * into the shutting down state where it will not accept any new cursors for registration.
      */
-    void shutdown();
+    void shutdown(OperationContext* opCtx);
 
     /**
      * Registers the given cursor with this manager, and returns the registered cursor's id, or
@@ -269,7 +297,8 @@ public:
      *
      * Does not block.
      */
-    StatusWith<CursorId> registerCursor(std::unique_ptr<ClusterClientCursor> cursor,
+    StatusWith<CursorId> registerCursor(OperationContext* opCtx,
+                                        std::unique_ptr<ClusterClientCursor> cursor,
                                         const NamespaceString& nss,
                                         CursorType cursorType,
                                         CursorLifetime cursorLifetime);
@@ -287,9 +316,11 @@ public:
      *
      * Does not block.
      */
+    enum AuthCheck { kCheckSession = true, kNoCheckSession = false };
     StatusWith<PinnedCursor> checkOutCursor(const NamespaceString& nss,
                                             CursorId cursorId,
-                                            OperationContext* txn);
+                                            OperationContext* opCtx,
+                                            AuthCheck checkSessionAuth = kCheckSession);
 
     /**
      * Informs the manager that the given cursor should be killed.  The cursor need not necessarily
@@ -323,13 +354,13 @@ public:
      * as 'kill pending'. Returns the number of cursors that were marked as inactive.
      *
      * If no other non-const methods are called simultaneously, it is guaranteed that this method
-     * will delete all non-pinned cursors marked as 'kill pending'.  Otherwise, no such guarantee is
+     * will delete all non-pinned cursors marked as 'kill pending'. Otherwise, no such guarantee is
      * made (this is due to the fact that the blocking kill for each cursor is performed outside of
      * the cursor manager lock).
      *
      * Can block.
      */
-    std::size_t reapZombieCursors();
+    std::size_t reapZombieCursors(OperationContext* opCtx);
 
     /**
      * Returns the number of open cursors on a ClusterCursorManager, broken down by type.
@@ -338,6 +369,24 @@ public:
      */
     Stats stats() const;
 
+    /**
+     * Appends sessions that have open cursors in this cursor manager to the given set of lsids.
+     */
+    void appendActiveSessions(LogicalSessionIdSet* lsids) const;
+
+    /**
+     * Returns a list of GenericCursors for all cursors in the cursor manager.
+     */
+    std::vector<GenericCursor> getAllCursors() const;
+
+    std::pair<Status, int> killCursorsWithMatchingSessions(OperationContext* opCtx,
+                                                           const SessionKiller::Matcher& matcher);
+
+    /**
+     * Returns a list of all open cursors for the given session.
+     */
+    stdx::unordered_set<CursorId> getCursorsForSession(LogicalSessionId lsid) const;
+
     /**
      * Returns the namespace associated with the given cursor id, by examining the 'namespace
      * prefix' portion of the cursor id.  A cursor with the given cursor id need not actually exist.
@@ -388,7 +437,7 @@ private:
      *
      * Not thread-safe.
      */
-    CursorEntry* getEntry_inlock(const NamespaceString& nss, CursorId cursorId);
+    CursorEntry* _getEntry(WithLock, NamespaceString const& nss, CursorId cursorId);
 
     /**
      * De-registers the given cursor, and returns an owned pointer to the underlying
@@ -399,8 +448,9 @@ private:
      *
      * Not thread-safe.
      */
-    StatusWith<std::unique_ptr<ClusterClientCursor>> detachCursor_inlock(const NamespaceString& nss,
-                                                                         CursorId cursorId);
+    StatusWith<std::unique_ptr<ClusterClientCursor>> _detachCursor(WithLock,
+                                                                   NamespaceString const& nss,
+                                                                   CursorId cursorId);
 
     /**
      * CursorEntry is a moveable, non-copyable container for a single cursor.
@@ -418,7 +468,8 @@ private:
             : _cursor(std::move(cursor)),
               _cursorType(cursorType),
               _cursorLifetime(cursorLifetime),
-              _lastActive(lastActive) {
+              _lastActive(lastActive),
+              _lsid(_cursor->getLsid()) {
             invariant(_cursor);
         }
 
@@ -449,6 +500,10 @@ private:
             return static_cast<bool>(_cursor);
         }
 
+        boost::optional<LogicalSessionId> getLsid() const {
+            return _lsid;
+        }
+
         /**
          * Releases the cursor from this entry.  If the cursor has already been released, returns
          * null.
@@ -482,9 +537,10 @@ private:
         std::unique_ptr<ClusterClientCursor> _cursor;
         bool _killPending = false;
         bool _isInactive = false;
-        CursorType _cursorType = CursorType::NamespaceNotSharded;
+        CursorType _cursorType = CursorType::SingleTarget;
         CursorLifetime _cursorLifetime = CursorLifetime::Mortal;
         Date_t _lastActive;
+        boost::optional<LogicalSessionId> _lsid;
     };
 
     /**
