@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -39,25 +41,25 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/connpool.h"
 #include "mongo/client/read_preference.h"
+#include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/query/getmore_request.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/platform/overflow_arithmetic.h"
-#include "mongo/rpc/metadata/server_selection_metadata.h"
-#include "mongo/s/catalog/catalog_cache.h"
-#include "mongo/s/chunk_manager.h"
+#include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/config.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/query/cluster_client_cursor_impl.h"
 #include "mongo/s/query/cluster_cursor_manager.h"
+#include "mongo/s/query/establish_cursors.h"
 #include "mongo/s/query/store_possible_cursor.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
+#include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
@@ -150,16 +152,17 @@ StatusWith<std::unique_ptr<QueryRequest>> transformQueryForShards(const QueryReq
     return std::move(newQR);
 }
 
-StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
+StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* opCtx,
                                              const CanonicalQuery& query,
                                              const ReadPreferenceSetting& readPref,
                                              ChunkManager* chunkManager,
                                              std::shared_ptr<Shard> primary,
                                              std::vector<BSONObj>* results,
                                              BSONObj* viewDefinition) {
-    auto shardRegistry = Grid::get(txn)->shardRegistry();
+    auto shardRegistry = Grid::get(opCtx)->shardRegistry();
 
     // Get the set of shards on which we will run the query.
+
     std::vector<std::shared_ptr<Shard>> shards;
     if (primary) {
         shards.emplace_back(std::move(primary));
@@ -167,13 +170,13 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
         invariant(chunkManager);
 
         std::set<ShardId> shardIds;
-        chunkManager->getShardIdsForQuery(txn,
+        chunkManager->getShardIdsForQuery(opCtx,
                                           query.getQueryRequest().getFilter(),
                                           query.getQueryRequest().getCollation(),
                                           &shardIds);
 
         for (auto id : shardIds) {
-            auto shardStatus = shardRegistry->getShard(txn, id);
+            auto shardStatus = shardRegistry->getShard(opCtx, id);
             if (!shardStatus.isOK()) {
                 return shardStatus.getStatus();
             }
@@ -181,14 +184,17 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
         }
     }
 
-    ClusterClientCursorParams params(query.nss(), readPref);
+    // Construct the query and parameters.
+
+    ClusterClientCursorParams params(
+        query.nss(),
+        AuthorizationSession::get(opCtx->getClient())->getAuthenticatedUserNames(),
+        readPref);
     params.limit = query.getQueryRequest().getLimit();
     params.batchSize = query.getQueryRequest().getEffectiveBatchSize();
     params.skip = query.getQueryRequest().getSkip();
-    params.isTailable = query.getQueryRequest().isTailable();
-    params.isAwaitData = query.getQueryRequest().isAwaitData();
+    params.tailableMode = query.getQueryRequest().getTailableMode();
     params.isAllowPartialResults = query.getQueryRequest().isAllowPartialResults();
-    params.txn = txn;
 
     // This is the batchSize passed to each subsequent getMore command issued by the cursor. We
     // usually use the batchSize associated with the initial find, but as it is illegal to send a
@@ -205,19 +211,19 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
     }
 
     // Tailable cursors can't have a sort, which should have already been validated.
-    invariant(params.sort.isEmpty() || !params.isTailable);
+    invariant(params.sort.isEmpty() || !query.getQueryRequest().isTailable());
 
     const auto qrToForward = transformQueryForShards(query.getQueryRequest());
     if (!qrToForward.isOK()) {
         return qrToForward.getStatus();
     }
 
-    // Use read pref to target a particular host from each shard. Also construct the find command
-    // that we will forward to each shard.
+    // Construct the find command that we will use to establish cursors, attaching the shardVersion.
+
+    std::vector<std::pair<ShardId, BSONObj>> requests;
     for (const auto& shard : shards) {
         invariant(!shard->isConfig() || shard->getConnString().type() != ConnectionString::INVALID);
 
-        // Build the find command, and attach shard version if necessary.
         BSONObjBuilder cmdBuilder;
         qrToForward.getValue()->asFindCommand(&cmdBuilder);
 
@@ -229,16 +235,50 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
             version.appendForCommands(&cmdBuilder);
         }
 
-        params.remotes.emplace_back(shard->getId(), cmdBuilder.obj());
+        requests.emplace_back(shard->getId(), cmdBuilder.obj());
     }
 
+    // Establish the cursors with a consistent shardVersion across shards.
+
+    auto swCursors = establishCursors(opCtx,
+                                      Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(),
+                                      query.nss(),
+                                      readPref,
+                                      requests,
+                                      query.getQueryRequest().isAllowPartialResults(),
+                                      viewDefinition);
+    if (!swCursors.isOK()) {
+        // Make sure a viewDefinition was set if the find was on a view.
+        if (ErrorCodes::CommandOnShardedViewNotSupportedOnMongod == swCursors.getStatus().code()) {
+            if (!viewDefinition) {
+                return {ErrorCodes::InternalError,
+                        str::stream() << "Missing resolved view definition, but remote returned "
+                                      << ErrorCodes::errorString(swCursors.getStatus().code())};
+            }
+        }
+        return swCursors.getStatus();
+    }
+
+    // Determine whether the cursor we may eventually register will be single- or multi-target.
+
+    const auto cursorType = swCursors.getValue().size() > 1
+        ? ClusterCursorManager::CursorType::MultiTarget
+        : ClusterCursorManager::CursorType::SingleTarget;
+
+    // Transfer the established cursors to a ClusterClientCursor.
+
+    params.remotes = std::move(swCursors.getValue());
     auto ccc = ClusterClientCursorImpl::make(
-        Grid::get(txn)->getExecutorPool()->getArbitraryExecutor(), std::move(params));
+        opCtx, Grid::get(opCtx)->getExecutorPool()->getArbitraryExecutor(), std::move(params));
+
+    // Retrieve enough data from the ClusterClientCursor for the first batch of results.
 
     auto cursorState = ClusterCursorManager::CursorState::NotExhausted;
     int bytesBuffered = 0;
+
     while (!FindCommon::enoughForFirstBatch(query.getQueryRequest(), results->size())) {
-        auto next = ccc->next();
+        auto next = ccc->next(RouterExecStage::ExecContext::kInitialFind);
+
         if (!next.isOK()) {
             return next.getStatus();
         }
@@ -254,14 +294,6 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
             break;
         }
 
-        if (next.getValue().getViewDefinition()) {
-            if (viewDefinition) {
-                *viewDefinition = BSON("resolvedView" << *next.getValue().getViewDefinition());
-            }
-            return {ErrorCodes::CommandOnShardedViewNotSupportedOnMongod,
-                    "Find must be transformed for view and run against base collection"};
-        }
-
         auto nextObj = *next.getValue().getResult();
 
         // If adding this object will cause us to exceed the message size limit, then we stash it
@@ -277,6 +309,8 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
         results->push_back(std::move(nextObj));
     }
 
+    ccc->detachFromOperationContext();
+
     if (!query.getQueryRequest().wantMore() && !ccc->isTailable()) {
         cursorState = ClusterCursorManager::CursorState::Exhausted;
     }
@@ -287,22 +321,21 @@ StatusWith<CursorId> runQueryWithoutRetrying(OperationContext* txn,
         return CursorId(0);
     }
 
-    // Register the cursor with the cursor manager.
-    auto cursorManager = Grid::get(txn)->getCursorManager();
-    const auto cursorType = chunkManager ? ClusterCursorManager::CursorType::NamespaceSharded
-                                         : ClusterCursorManager::CursorType::NamespaceNotSharded;
+    // Register the cursor with the cursor manager for subsequent getMore's.
+
+    auto cursorManager = Grid::get(opCtx)->getCursorManager();
     const auto cursorLifetime = query.getQueryRequest().isNoCursorTimeout()
         ? ClusterCursorManager::CursorLifetime::Immortal
         : ClusterCursorManager::CursorLifetime::Mortal;
     return cursorManager->registerCursor(
-        ccc.releaseCursor(), query.nss(), cursorType, cursorLifetime);
+        opCtx, ccc.releaseCursor(), query.nss(), cursorType, cursorLifetime);
 }
 
 }  // namespace
 
 const size_t ClusterFind::kMaxStaleConfigRetries = 10;
 
-StatusWith<CursorId> ClusterFind::runQuery(OperationContext* txn,
+StatusWith<CursorId> ClusterFind::runQuery(OperationContext* opCtx,
                                            const CanonicalQuery& query,
                                            const ReadPreferenceSetting& readPref,
                                            std::vector<BSONObj>* results,
@@ -318,28 +351,34 @@ StatusWith<CursorId> ClusterFind::runQuery(OperationContext* txn,
                               << query.getQueryRequest().getProj()};
     }
 
-    auto dbConfig = Grid::get(txn)->catalogCache()->getDatabase(txn, query.nss().db().toString());
-    if (dbConfig.getStatus() == ErrorCodes::NamespaceNotFound) {
-        // If the database doesn't exist, we successfully return an empty result set without
-        // creating a cursor.
-        return CursorId(0);
-    } else if (!dbConfig.isOK()) {
-        return dbConfig.getStatus();
-    }
-
-    std::shared_ptr<ChunkManager> chunkManager;
-    std::shared_ptr<Shard> primary;
-    dbConfig.getValue()->getChunkManagerOrPrimary(txn, query.nss().ns(), chunkManager, primary);
+    auto const catalogCache = Grid::get(opCtx)->catalogCache();
 
     // Re-target and re-send the initial find command to the shards until we have established the
     // shard version.
     for (size_t retries = 1; retries <= kMaxStaleConfigRetries; ++retries) {
-        auto cursorId = runQueryWithoutRetrying(
-            txn, query, readPref, chunkManager.get(), std::move(primary), results, viewDefinition);
+        auto routingInfoStatus = catalogCache->getCollectionRoutingInfo(opCtx, query.nss());
+        if (routingInfoStatus == ErrorCodes::NamespaceNotFound) {
+            // If the database doesn't exist, we successfully return an empty result set without
+            // creating a cursor.
+            return CursorId(0);
+        } else if (!routingInfoStatus.isOK()) {
+            return routingInfoStatus.getStatus();
+        }
+
+        auto& routingInfo = routingInfoStatus.getValue();
+
+        auto cursorId = runQueryWithoutRetrying(opCtx,
+                                                query,
+                                                readPref,
+                                                routingInfo.cm().get(),
+                                                routingInfo.primary(),
+                                                results,
+                                                viewDefinition);
         if (cursorId.isOK()) {
             return cursorId;
         }
-        auto status = std::move(cursorId.getStatus());
+
+        const auto& status = cursorId.getStatus();
 
         if (!ErrorCodes::isStaleShardingError(status.code()) &&
             status != ErrorCodes::ShardNotFound) {
@@ -353,20 +392,7 @@ StatusWith<CursorId> ClusterFind::runQuery(OperationContext* txn,
                << " on attempt " << retries << " of " << kMaxStaleConfigRetries << ": "
                << redact(status);
 
-        const bool staleEpoch = (status == ErrorCodes::StaleEpoch);
-        if (staleEpoch) {
-            if (!dbConfig.getValue()->reload(txn)) {
-                // If the reload failed that means the database wasn't found, so successfully return
-                // an empty result set without creating a cursor.
-                return CursorId(0);
-            }
-        }
-        chunkManager =
-            dbConfig.getValue()->getChunkManagerIfExists(txn, query.nss().ns(), true, staleEpoch);
-        if (!chunkManager) {
-            dbConfig.getValue()->getChunkManagerOrPrimary(
-                txn, query.nss().ns(), chunkManager, primary);
-        }
+        catalogCache->onStaleConfigError(std::move(routingInfo));
     }
 
     return {ErrorCodes::StaleShardVersion,
@@ -374,11 +400,11 @@ StatusWith<CursorId> ClusterFind::runQuery(OperationContext* txn,
                           << " times without successfully establishing shard version."};
 }
 
-StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* txn,
+StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* opCtx,
                                                    const GetMoreRequest& request) {
-    auto cursorManager = Grid::get(txn)->getCursorManager();
+    auto cursorManager = Grid::get(opCtx)->getCursorManager();
 
-    auto pinnedCursor = cursorManager->checkOutCursor(request.nss, request.cursorid, txn);
+    auto pinnedCursor = cursorManager->checkOutCursor(request.nss, request.cursorid, opCtx);
     if (!pinnedCursor.isOK()) {
         return pinnedCursor.getStatus();
     }
@@ -388,11 +414,36 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* txn,
     while (MONGO_FAIL_POINT(keepCursorPinnedDuringGetMore)) {
     }
 
-    if (request.awaitDataTimeout) {
-        auto status = pinnedCursor.getValue().setAwaitDataTimeout(*request.awaitDataTimeout);
-        if (!status.isOK()) {
-            return status;
-        }
+    // A user can only call getMore on their own cursor. If there were multiple users authenticated
+    // when the cursor was created, then at least one of them must be authenticated in order to run
+    // getMore on the cursor.
+    if (!AuthorizationSession::get(opCtx->getClient())
+             ->isCoauthorizedWith(pinnedCursor.getValue().getAuthenticatedUsers())) {
+        // Return the cursor so that it's not killed.
+        pinnedCursor.getValue().returnCursor(ClusterCursorManager::CursorState::NotExhausted);
+        return {ErrorCodes::Unauthorized,
+                str::stream() << "cursor id " << request.cursorid
+                              << " was not created by the authenticated user"};
+    }
+
+    if (auto readPref = pinnedCursor.getValue().getReadPreference()) {
+        ReadPreferenceSetting::get(opCtx) = *readPref;
+    }
+    if (pinnedCursor.getValue().isTailableAndAwaitData()) {
+        // A maxTimeMS specified on a tailable, awaitData cursor is special. Instead of imposing a
+        // deadline on the operation, it is used to communicate how long the server should wait for
+        // new results. Here we clear any deadline set during command processing and track the
+        // deadline instead via the 'waitForInsertsDeadline' decoration. This deadline defaults to
+        // 1 second if the user didn't specify a maxTimeMS.
+        opCtx->clearDeadline();
+        auto timeout = request.awaitDataTimeout.value_or(Milliseconds{1000});
+        awaitDataState(opCtx).waitForInsertsDeadline =
+            opCtx->getServiceContext()->getPreciseClockSource()->now() + timeout;
+
+        invariant(pinnedCursor.getValue().setAwaitDataTimeout(timeout).isOK());
+    } else if (request.awaitDataTimeout) {
+        return {ErrorCodes::BadValue,
+                "maxTimeMS can only be used with getMore for tailable, awaitData cursors"};
     }
 
     std::vector<BSONObj> batch;
@@ -400,15 +451,44 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* txn,
     long long batchSize = request.batchSize.value_or(0);
     long long startingFrom = pinnedCursor.getValue().getNumReturnedSoFar();
     auto cursorState = ClusterCursorManager::CursorState::NotExhausted;
+
+    pinnedCursor.getValue().reattachToOperationContext(opCtx);
+
+    // A pinned cursor will not be destroyed immediately if an exception is thrown. Instead it will
+    // be marked as killed, then reaped by a background thread later. If this happens, we want to be
+    // sure the cursor does not have a pointer to this OperationContext, since it will be destroyed
+    // as soon as we return, but the cursor will live on a bit longer.
+    ScopeGuard cursorDetach =
+        MakeGuard([&pinnedCursor]() { pinnedCursor.getValue().detachFromOperationContext(); });
+
     while (!FindCommon::enoughForGetMore(batchSize, batch.size())) {
-        auto next = pinnedCursor.getValue().next();
+        auto context = batch.empty()
+            ? RouterExecStage::ExecContext::kGetMoreNoResultsYet
+            : RouterExecStage::ExecContext::kGetMoreWithAtLeastOneResultInBatch;
+
+        StatusWith<ClusterQueryResult> next =
+            Status{ErrorCodes::InternalError, "uninitialized cluster query result"};
+        try {
+            next = pinnedCursor.getValue().next(context);
+        } catch (const ExceptionFor<ErrorCodes::CloseChangeStream>&) {
+            // This exception is thrown when a $changeStream stage encounters an event
+            // that invalidates the cursor. We should close the cursor and return without
+            // error.
+            cursorState = ClusterCursorManager::CursorState::Exhausted;
+            break;
+        }
+
         if (!next.isOK()) {
             return next.getStatus();
         }
 
         if (next.getValue().isEOF()) {
-            // We reached end-of-stream.
-            if (!pinnedCursor.getValue().isTailable()) {
+            // We reached end-of-stream. If the cursor is not tailable, then we mark it as
+            // exhausted. If it is tailable, usually we keep it open (i.e. "NotExhausted") even when
+            // we reach end-of-stream. However, if all the remote cursors are exhausted, there is no
+            // hope of returning data and thus we need to close the mongos cursor as well.
+            if (!pinnedCursor.getValue().isTailable() ||
+                pinnedCursor.getValue().remotesExhausted()) {
                 cursorState = ClusterCursorManager::CursorState::Exhausted;
             }
             break;
@@ -427,7 +507,10 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* txn,
         batch.push_back(std::move(*next.getValue().getResult()));
     }
 
-    // Transfer ownership of the cursor back to the cursor manager.
+    // Upon successful completion, we need to detach from the operation and transfer ownership of
+    // the cursor back to the cursor manager.
+    cursorDetach.Dismiss();
+    pinnedCursor.getValue().detachFromOperationContext();
     pinnedCursor.getValue().returnCursor(cursorState);
 
     CursorId idToReturn = (cursorState == ClusterCursorManager::CursorState::Exhausted)
@@ -436,32 +519,4 @@ StatusWith<CursorResponse> ClusterFind::runGetMore(OperationContext* txn,
     return CursorResponse(request.nss, idToReturn, std::move(batch), startingFrom);
 }
 
-StatusWith<ReadPreferenceSetting> ClusterFind::extractUnwrappedReadPref(const BSONObj& cmdObj,
-                                                                        const bool isSlaveOk) {
-    BSONElement queryOptionsElt;
-    auto status = bsonExtractTypedField(
-        cmdObj, QueryRequest::kUnwrappedReadPrefField, BSONType::Object, &queryOptionsElt);
-    if (status.isOK()) {
-        // There must be a nested object containing the read preference if there is a queryOptions
-        // field.
-        BSONObj queryOptionsObj = queryOptionsElt.Obj();
-        invariant(queryOptionsObj[QueryRequest::kWrappedReadPrefField].type() == BSONType::Object);
-        BSONObj readPrefObj = queryOptionsObj[QueryRequest::kWrappedReadPrefField].Obj();
-
-        auto readPref = ReadPreferenceSetting::fromBSON(readPrefObj);
-        if (!readPref.isOK()) {
-            return readPref.getStatus();
-        }
-        return readPref.getValue();
-    } else if (status != ErrorCodes::NoSuchKey) {
-        return status;
-    }
-
-    // If there is no explicit read preference, the value we use depends on the setting of the slave
-    // ok bit.
-    ReadPreference pref =
-        isSlaveOk ? mongo::ReadPreference::SecondaryPreferred : mongo::ReadPreference::PrimaryOnly;
-    return ReadPreferenceSetting(pref, TagSet());
-}
-
 }  // namespace mongo
