@@ -1,29 +1,31 @@
+
 /**
- * Copyright (C) 2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- * This program is free software: you can redistribute it and/or  modify
- * it under the terms of the GNU Affero General Public License, version 3,
- * as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    Server Side Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
- * As a special exception, the copyright holders give permission to link the
- * code of portions of this program with the OpenSSL library under certain
- * conditions as described in each individual source file and distribute
- * linked combinations including the program with the OpenSSL library. You
- * must comply with the GNU Affero General Public License in all respects
- * for all of the code used other than as permitted herein. If you modify
- * file(s) with this exception, you may extend this exception to your
- * version of the file(s), but you are not obligated to do so. If you do not
- * wish to do so, delete this exception statement from your version. If you
- * delete this exception statement from all source files in the program,
- * then also delete it in the license file.
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
@@ -514,37 +516,31 @@ auto MozJSImplScope::_runSafely(ImplScopeFunction&& functionToRun) -> decltype(f
 }
 
 void MozJSImplScope::setNumber(const char* field, double val) {
-    _runSafely([this, &field, &val] { ObjectWrapper(_context, _global).setNumber(field, val); });
+    _runSafely([&] { ObjectWrapper(_context, _global).setNumber(field, val); });
 }
 
 void MozJSImplScope::setString(const char* field, StringData val) {
-    _runSafely([this, &field, &val] { ObjectWrapper(_context, _global).setString(field, val); });
+    _runSafely([&] { ObjectWrapper(_context, _global).setString(field, val); });
 }
 
 void MozJSImplScope::setBoolean(const char* field, bool val) {
-    _runSafely([this, &field, &val] { ObjectWrapper(_context, _global).setBoolean(field, val); });
+    _runSafely([&] { ObjectWrapper(_context, _global).setBoolean(field, val); });
 }
 
 void MozJSImplScope::setElement(const char* field, const BSONElement& e, const BSONObj& parent) {
-    _runSafely([this, &field, &e, &parent] {
-
-        ObjectWrapper(_context, _global).setBSONElement(field, e, parent, false);
-    });
+    _runSafely([&] { ObjectWrapper(_context, _global).setBSONElement(field, e, parent, false); });
 }
 
 void MozJSImplScope::setObject(const char* field, const BSONObj& obj, bool readOnly) {
-    _runSafely([this, &field, &obj, &readOnly] {
-
-        ObjectWrapper(_context, _global).setBSON(field, obj, readOnly);
-    });
+    _runSafely([&] { ObjectWrapper(_context, _global).setBSON(field, obj, readOnly); });
 }
 
 int MozJSImplScope::type(const char* field) {
-    return _runSafely([this, &field] { return ObjectWrapper(_context, _global).type(field); });
+    return _runSafely([&] { return ObjectWrapper(_context, _global).type(field); });
 }
 
 double MozJSImplScope::getNumber(const char* field) {
-    return _runSafely([this, &field] { return ObjectWrapper(_context, _global).getNumber(field); });
+    return _runSafely([&] { return ObjectWrapper(_context, _global).getNumber(field); });
 }
 
 int MozJSImplScope::getNumberInt(const char* field) {
@@ -563,7 +559,7 @@ Decimal128 MozJSImplScope::getNumberDecimal(const char* field) {
 }
 
 std::string MozJSImplScope::getString(const char* field) {
-    return _runSafely([this, &field] { return ObjectWrapper(_context, _global).getString(field); });
+    return _runSafely([&] { return ObjectWrapper(_context, _global).getString(field); });
 }
 
 bool MozJSImplScope::getBoolean(const char* field) {
@@ -572,13 +568,11 @@ bool MozJSImplScope::getBoolean(const char* field) {
 }
 
 BSONObj MozJSImplScope::getObject(const char* field) {
-    return _runSafely([this, &field] { return ObjectWrapper(_context, _global).getObject(field); });
+    return _runSafely([&] { return ObjectWrapper(_context, _global).getObject(field); });
 }
 
 void MozJSImplScope::newFunction(StringData raw, JS::MutableHandleValue out) {
-    MozJSEntry entry(this);
-
-    _MozJSCreateFunction(raw, std::move(out));
+    _runSafely([&] { _MozJSCreateFunction(raw, std::move(out)); });
 }
 
 void MozJSImplScope::_MozJSCreateFunction(StringData raw, JS::MutableHandleValue fun) {
@@ -639,16 +633,16 @@ bool hasFunctionIdentifier(StringData code) {
 }
 
 ScriptingFunction MozJSImplScope::_createFunction(const char* raw) {
-    MozJSEntry entry(this);
-
-    JS::RootedValue fun(_context);
-    _MozJSCreateFunction(raw, &fun);
-    _funcs.emplace_back(_context, fun.get());
-    return _funcs.size();
+    return _runSafely([&] {
+        JS::RootedValue fun(_context);
+        _MozJSCreateFunction(raw, &fun);
+        _funcs.emplace_back(_context, fun.get());
+        return _funcs.size();
+    });
 }
 
 void MozJSImplScope::setFunction(const char* field, const char* code) {
-    _runSafely([this, &field, &code] {
+    _runSafely([&] {
         JS::RootedValue fun(_context);
         _MozJSCreateFunction(code, &fun);
         ObjectWrapper(_context, _global).setValue(field, fun);
@@ -656,7 +650,7 @@ void MozJSImplScope::setFunction(const char* field, const char* code) {
 }
 
 void MozJSImplScope::rename(const char* from, const char* to) {
-    _runSafely([this, &from, &to] { ObjectWrapper(_context, _global).rename(from, to); });
+    _runSafely([&] { ObjectWrapper(_context, _global).rename(from, to); });
 }
 
 int MozJSImplScope::invoke(ScriptingFunction func,
@@ -666,62 +660,62 @@ int MozJSImplScope::invoke(ScriptingFunction func,
                            bool ignoreReturn,
                            bool readOnlyArgs,
                            bool readOnlyRecv) {
-    MozJSEntry entry(this);
+    return _runSafely([&] {
+        auto funcValue = _funcs[func - 1];
+        JS::RootedValue result(_context);
 
-    auto funcValue = _funcs[func - 1];
-    JS::RootedValue result(_context);
+        const int nargs = argsObject ? argsObject->nFields() : 0;
 
-    const int nargs = argsObject ? argsObject->nFields() : 0;
+        JS::AutoValueVector args(_context);
 
-    JS::AutoValueVector args(_context);
+        if (nargs) {
+            BSONObjIterator it(*argsObject);
+            for (int i = 0; i < nargs; i++) {
+                BSONElement next = it.next();
 
-    if (nargs) {
-        BSONObjIterator it(*argsObject);
-        for (int i = 0; i < nargs; i++) {
-            BSONElement next = it.next();
+                JS::RootedValue value(_context);
+                ValueReader(_context, &value).fromBSONElement(next, *argsObject, readOnlyArgs);
 
-            JS::RootedValue value(_context);
-            ValueReader(_context, &value).fromBSONElement(next, *argsObject, readOnlyArgs);
-
-            args.append(value);
-        }
-    }
-
-    JS::RootedValue smrecv(_context);
-    if (recv)
-        ValueReader(_context, &smrecv).fromBSON(*recv, nullptr, readOnlyRecv);
-    else
-        smrecv.setObjectOrNull(_global);
-
-    if (timeoutMs)
-        _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
-    else {
-        _engine->getDeadlineMonitor().startDeadline(this, -1);
-    }
-
-    JS::RootedValue out(_context);
-    JS::RootedObject obj(_context, smrecv.toObjectOrNull());
-
-    bool success = JS::Call(_context, obj, funcValue, args, &out);
-
-    _engine->getDeadlineMonitor().stopDeadline(this);
-
-    _checkErrorState(success);
-
-    if (!ignoreReturn) {
-        // must validate the handle because TerminateExecution may have
-        // been thrown after the above checks
-        if (out.isObject() && _nativeFunctionProto.instanceOf(out)) {
-            warning() << "storing native function as return value";
-            _lastRetIsNativeCode = true;
-        } else {
-            _lastRetIsNativeCode = false;
+                args.append(value);
+            }
         }
 
-        ObjectWrapper(_context, _global).setValue(kInvokeResult, out);
-    }
+        JS::RootedValue smrecv(_context);
+        if (recv)
+            ValueReader(_context, &smrecv).fromBSON(*recv, nullptr, readOnlyRecv);
+        else
+            smrecv.setObjectOrNull(_global);
 
-    return 0;
+        if (timeoutMs)
+            _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
+        else {
+            _engine->getDeadlineMonitor().startDeadline(this, -1);
+        }
+
+        JS::RootedValue out(_context);
+        JS::RootedObject obj(_context, smrecv.toObjectOrNull());
+
+        bool success = JS::Call(_context, obj, funcValue, args, &out);
+
+        _engine->getDeadlineMonitor().stopDeadline(this);
+
+        _checkErrorState(success);
+
+        if (!ignoreReturn) {
+            // must validate the handle because TerminateExecution may have
+            // been thrown after the above checks
+            if (out.isObject() && _nativeFunctionProto.instanceOf(out)) {
+                warning() << "storing native function as return value";
+                _lastRetIsNativeCode = true;
+            } else {
+                _lastRetIsNativeCode = false;
+            }
+
+            ObjectWrapper(_context, _global).setValue(kInvokeResult, out);
+        }
+
+        return 0;
+    });
 }
 
 bool MozJSImplScope::exec(StringData code,
@@ -730,45 +724,45 @@ bool MozJSImplScope::exec(StringData code,
                           bool reportError,
                           bool assertOnError,
                           int timeoutMs) {
-    MozJSEntry entry(this);
+    return _runSafely([&] {
+        JS::CompileOptions co(_context);
+        setCompileOptions(&co);
+        co.setFileAndLine(name.c_str(), 1);
+        JS::RootedScript script(_context);
 
-    JS::CompileOptions co(_context);
-    setCompileOptions(&co);
-    co.setFileAndLine(name.c_str(), 1);
-    JS::RootedScript script(_context);
+        bool success = JS::Compile(_context, co, code.rawData(), code.size(), &script);
 
-    bool success = JS::Compile(_context, co, code.rawData(), code.size(), &script);
+        if (_checkErrorState(success, reportError, assertOnError))
+            return false;
 
-    if (_checkErrorState(success, reportError, assertOnError))
-        return false;
+        if (timeoutMs) {
+            _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
+        } else {
+            _engine->getDeadlineMonitor().startDeadline(this, -1);
+        }
 
-    if (timeoutMs) {
-        _engine->getDeadlineMonitor().startDeadline(this, timeoutMs);
-    } else {
-        _engine->getDeadlineMonitor().startDeadline(this, -1);
-    }
+        JS::RootedValue out(_context);
 
-    JS::RootedValue out(_context);
+        success = JS_ExecuteScript(_context, script, &out);
 
-    success = JS_ExecuteScript(_context, script, &out);
+        _engine->getDeadlineMonitor().stopDeadline(this);
 
-    _engine->getDeadlineMonitor().stopDeadline(this);
+        if (_checkErrorState(success, reportError, assertOnError))
+            return false;
 
-    if (_checkErrorState(success, reportError, assertOnError))
-        return false;
+        ObjectWrapper(_context, _global).setValue(kExecResult, out);
 
-    ObjectWrapper(_context, _global).setValue(kExecResult, out);
+        if (printResult && !out.isUndefined()) {
+            // appears to only be used by shell
+            std::cout << ValueWriter(_context, out).toString() << std::endl;
+        }
 
-    if (printResult && !out.isUndefined()) {
-        // appears to only be used by shell
-        std::cout << ValueWriter(_context, out).toString() << std::endl;
-    }
-
-    return true;
+        return true;
+    });
 }
 
 void MozJSImplScope::injectNative(const char* field, NativeFunction func, void* data) {
-    _runSafely([this, &field, &func, &data] {
+    _runSafely([&] {
         JS::RootedObject obj(_context);
 
         NativeFunctionInfo::make(_context, &obj, func, data);
@@ -820,7 +814,6 @@ void MozJSImplScope::localConnectForDbEval(OperationContext* opCtx, const char*
 }
 
 void MozJSImplScope::externalSetup() {
-
     _runSafely([&] {
         if (_connectState == ConnectState::External)
             return;
