@@ -1,33 +1,39 @@
-/*
- *    Copyright (C) 2010 10gen Inc.
+
+/**
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
+#include "mongo/platform/basic.h"
+
+#include "mongo/bson/json.h"
 #include "mongo/scripting/engine.h"
 #include "mongo/util/md5.hpp"
+#include "mongo/util/time_support.h"
 
 namespace mongo {
 
@@ -57,6 +63,20 @@ static BSONObj native_sleep(const mongo::BSONObj& args, void* data) {
     return b.obj();
 }
 
+static BSONObj native_tostrictjson(const mongo::BSONObj& args, void* data) {
+    uassert(40275,
+            "tostrictjson takes a single BSON object argument, and on optional boolean argument "
+            "for prettyPrint -- tostrictjson(obj, prettyPrint = false)",
+            args.nFields() >= 1 && args.firstElement().isABSONObj() &&
+                (args.nFields() == 1 || (args.nFields() == 2 && args["1"].isBoolean())));
+
+    bool prettyPrint = false;
+    if (args.nFields() == 2) {
+        prettyPrint = args["1"].boolean();
+    }
+    return BSON("" << tojson(args.firstElement().embeddedObject(), Strict, prettyPrint));
+}
+
 // ---------------------------------
 // ---- installer           --------
 // ---------------------------------
@@ -64,6 +84,7 @@ static BSONObj native_sleep(const mongo::BSONObj& args, void* data) {
 void installGlobalUtils(Scope& scope) {
     scope.injectNative("hex_md5", native_hex_md5);
     scope.injectNative("sleep", native_sleep);
+    scope.injectNative("tostrictjson", native_tostrictjson);
 }
 
 }  // namespace mongo
