@@ -1,29 +1,31 @@
-/*
- *    Copyright (C) 2013 10gen Inc.
+
+/**
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #pragma once
@@ -62,24 +64,21 @@ struct ShellGlobalParams {
     bool nodb;
     bool norc;
     bool nojit = false;
-    bool javascriptProtection = false;
+    bool javascriptProtection = true;
 
     std::string script;
 
-    bool autoKillOp;
-    bool useWriteCommandsDefault;
-    std::string writeMode;
+    bool autoKillOp = false;
+    bool useWriteCommandsDefault = true;
 
-    std::string readMode;
+    std::string writeMode = "commands";
+    std::string readMode = "compatibility";
+    bool shouldRetryWrites = false;
+    bool shouldUseImplicitSessions = true;
 
-    boost::optional<rpc::ProtocolSet> rpcProtocols;
+    boost::optional<rpc::ProtocolSet> rpcProtocols = boost::none;
 
-    ShellGlobalParams()
-        : autoKillOp(false),
-          useWriteCommandsDefault(true),
-          writeMode("commands"),
-          readMode("compatibility"),
-          rpcProtocols() {}
+    int jsHeapLimitMB = 0;
 };
 
 extern ShellGlobalParams shellGlobalParams;
@@ -97,4 +96,6 @@ bool handlePreValidationMongoShellOptions(const moe::Environment& params,
                                           const std::vector<std::string>& args);
 
 Status storeMongoShellOptions(const moe::Environment& params, const std::vector<std::string>& args);
+
+void redactPasswordOptions(int argc, char** argv);
 }
