@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2016 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -52,11 +54,25 @@ namespace mongo {
 namespace transport {
 
 TransportLayerLegacy::Options::Options(const ServerGlobalParams* params)
-    : port(params->port), ipList(params->bind_ip) {}
+    : port(params->port), ipList(params->bind_ips) {}
 
 TransportLayerLegacy::ListenerLegacy::ListenerLegacy(const TransportLayerLegacy::Options& opts,
                                                      NewConnectionCb callback)
-    : Listener("", opts.ipList, opts.port, getGlobalServiceContext(), true),
+    : Listener("",
+               [](const std::vector<std::string>& ips) -> std::string {
+                   // convert IP vector back to string for compatibility with legacy networking code
+                   // forgive me father
+                   StringBuilder ip_str;
+                   StringData comma;
+                   for (const auto& ip : ips) {
+                       ip_str << comma << ip;
+                       comma = ",";
+                   }
+                   return ip_str.str();
+               }(opts.ipList),
+               opts.port,
+               getGlobalServiceContext(),
+               true),
       _accepted(std::move(callback)) {}
 
 void TransportLayerLegacy::ListenerLegacy::accepted(std::unique_ptr<AbstractMessagingPort> mp) {
@@ -80,7 +96,8 @@ std::shared_ptr<TransportLayerLegacy::LegacySession> TransportLayerLegacy::Legac
 
 TransportLayerLegacy::LegacySession::LegacySession(std::unique_ptr<AbstractMessagingPort> amp,
                                                    TransportLayerLegacy* tl)
-    : _remote(amp->remoteAddr()),
+    : Session(amp->connectionId()),
+      _remote(amp->remoteAddr()),
       _local(amp->localAddr()),
       _tl(tl),
       _connection(stdx::make_unique<Connection>(std::move(amp))) {}
