@@ -1,29 +1,31 @@
+
 /**
- * Copyright (C) 2016 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- * This program is free software: you can redistribute it and/or  modify
- * it under the terms of the GNU Affero General Public License, version 3,
- * as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Affero General Public License for more details.
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    Server Side Public License for more details.
  *
- * You should have received a copy of the GNU Affero General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
- * As a special exception, the copyright holders give permission to link the
- * code of portions of this program with the OpenSSL library under certain
- * conditions as described in each individual source file and distribute
- * linked combinations including the program with the OpenSSL library. You
- * must comply with the GNU Affero General Public License in all respects
- * for all of the code used other than as permitted herein. If you modify
- * file(s) with this exception, you may extend this exception to your
- * version of the file(s), but you are not obligated to do so. If you do not
- * wish to do so, delete this exception statement from your version. If you
- * delete this exception statement from all source files in the program,
- * then also delete it in the license file.
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kFTDC
@@ -406,6 +408,101 @@ Status parseProcMemInfoFile(StringData filename,
     return parseProcMemInfo(keys, swString.getValue(), builder);
 }
 
+//
+// Here is an example of the type of string it supports (long lines elided for clarity).
+// > cat /proc/net/netstat
+// TcpExt: SyncookiesSent SyncookiesRecv SyncookiesFailed ...
+// TcpExt: 3437 5938 13368 ...
+// IpExt: InNoRoutes InTruncatedPkts InMcastPkts ...
+// IpExt: 999 1 4819969 ...
+//
+// Parser assumes file consists of alternating lines of keys and values
+// key and value lines consist of space-separated tokens
+// first token is a key prefix that is prepended in the output to each key
+// all prefixed keys and corresponding values are copied to output as-is
+//
+
+Status parseProcNetstat(const std::vector<StringData>& keys,
+                        StringData data,
+                        BSONObjBuilder* builder) {
+
+    using string_split_iterator = boost::split_iterator<StringData::const_iterator>;
+
+    string_split_iterator keysIt;
+    bool foundKeys = false;
+
+    // Split the file by lines.
+    uint32_t lineNum = 0;
+    for (string_split_iterator
+             lineIt = string_split_iterator(
+                 data.begin(),
+                 data.end(),
+                 boost::token_finder([](char c) { return c == '\n'; }, boost::token_compress_on));
+         lineIt != string_split_iterator();
+         ++lineIt, ++lineNum) {
+
+        if (lineNum % 2 == 0) {
+
+            // even numbered lines are keys
+            keysIt = string_split_iterator(
+                (*lineIt).begin(),
+                (*lineIt).end(),
+                boost::token_finder([](char c) { return c == ' '; }, boost::token_compress_on));
+
+        } else {
+
+            // odd numbered lines are values
+            string_split_iterator valuesIt = string_split_iterator(
+                (*lineIt).begin(),
+                (*lineIt).end(),
+                boost::token_finder([](char c) { return c == ' '; }, boost::token_compress_on));
+
+            StringData prefix;
+
+            // iterate over the keys and values in parallel
+            for (uint32_t keyNum = 0;
+                 keysIt != string_split_iterator() && valuesIt != string_split_iterator();
+                 ++keysIt, ++valuesIt, ++keyNum) {
+
+                if (keyNum == 0) {
+
+                    // first token is a prefix to be applied to remaining keys
+                    prefix = StringData((*keysIt).begin(), (*keysIt).end());
+
+                    // ignore line if prefix isn't in requested list
+                    if (!keys.empty() && std::find(keys.begin(), keys.end(), prefix) == keys.end())
+                        break;
+
+                } else {
+
+                    // remaining tokens are key/value pairs
+                    StringData key((*keysIt).begin(), (*keysIt).end());
+                    StringData stringValue((*valuesIt).begin(), (*valuesIt).end());
+                    uint64_t value;
+                    if (parseNumberFromString(stringValue, &value).isOK()) {
+                        builder->appendNumber(prefix.toString() + key.toString(), value);
+                        foundKeys = true;
+                    }
+                }
+            }
+        }
+    }
+
+    return foundKeys ? Status::OK()
+                     : Status(ErrorCodes::NoSuchKey, "Failed to find any keys in netstats string");
+}
+
+Status parseProcNetstatFile(const std::vector<StringData>& keys,
+                            StringData filename,
+                            BSONObjBuilder* builder) {
+    auto swString = readFileAsString(filename);
+    if (!swString.isOK()) {
+        return swString.getStatus();
+    }
+    return parseProcNetstat(keys, swString.getValue(), builder);
+}
+
+
 // Here is an example of the type of string it supports:
 //
 // For more information, see:
